from __future__ import annotations

import json
import os
import logging
from collections.abc import Iterable
from typing import TYPE_CHECKING, Any, cast

from schema import SchemaError  # type: ignore

from checkov.common.images.image_referencer import ImageReferencer, Image
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.util.consts import START_LINE, END_LINE
from checkov.github_actions.checks.registry import registry
from checkov.github_actions.schema_validator import schema
from checkov.yaml_doc.runner import Runner as YamlRunner
if TYPE_CHECKING:
    from checkov.common.checks.base_check_registry import BaseCheckRegistry

WORKFLOW_DIRECTORY = ".github/workflows/"


class Runner(YamlRunner, ImageReferencer):
    check_type = CheckType.GITHUB_ACTIONS  # noqa: CCE003  # a static attribute

    def __init__(self):
        super().__init__()

    def require_external_checks(self) -> bool:
        return False

    def import_registry(self) -> BaseCheckRegistry:
        return registry

    def _parse_file(
        self, f: str, file_content: str | None = None
    ) -> tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
        if self.is_workflow_file(f):
            entity_schema: tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] = super()._parse_file(f)
            if entity_schema and Runner.is_schema_valid(entity_schema[0]):
                return entity_schema
        return None

    def is_workflow_file(self, file_path: str) -> bool:
        """
        :return: True if the file mentioned is in a github action workflow directory and is a YAML file. Otherwise: False
        """
        abspath = os.path.abspath(file_path)
        return WORKFLOW_DIRECTORY in abspath and abspath.endswith(("yml", "yaml"))

    def included_paths(self) -> Iterable[str]:
        return [".github"]

    def get_resource(self, file_path: str, key: str, supported_entities: Iterable[str], definitions: dict[str, Any] | None = None) -> str:
        if not definitions:
            return key

        potential_job_name = key.split('.')[1]
        if potential_job_name != '*':
            new_key = f'jobs.{potential_job_name}'
        else:
            start_line, end_line = Runner.get_start_and_end_lines(key)
            job_name = Runner.resolve_job_name(definitions, start_line, end_line)
            step_name = Runner.resolve_step_name(definitions["jobs"][job_name], start_line, end_line)
            new_key = f'jobs.{job_name}.steps.{step_name}'
        return new_key

    @staticmethod
    def generate_resource_key(definition: dict[str, Any], start_line: int, end_line: int) -> str:
        """
        Generate resource key without the previous format of key (needed in get_resource)
        """
        jobs_dict: dict[str, Any] = definition.get("jobs", {})
        for job_name, job in jobs_dict.items():
            if not isinstance(job, dict):
                continue

            if job[START_LINE] <= start_line <= end_line <= job[END_LINE]:
                return f'jobs.{job_name}'

        return ''

    def get_images(self, file_path: str) -> set[Image]:
        """
        Get container images mentioned in a file
        :param file_path: File to be inspected
        GitHub actions workflow file can have a job run within a container.

        in the following sample file we can see a node:14.16 image:

        # jobs:
        #   my_job:
        #     container:
        #       image: node:14.16
        #       env:
        #         NODE_ENV: development
        #       ports:
        #         - 80
        #       volumes:
        #         - my_docker_volume:/volume_mount
        #       options: --cpus 1
        Source: https://docs.github.com/en/actions/using-workflows/workflow-syntax-for-github-actions#example-defining-credentials-for-a-container-registry

        :return: List of container image classes ids mentioned in the file.
        """

        images: set[Image] = set()
        parsed_file = self._parse_file(file_path)

        if not parsed_file:
            return images

        workflow, workflow_line_numbers = parsed_file

        if not isinstance(workflow, dict):
            # make type checking happy
            return images

        jobs = workflow.get("jobs", {})
        for job_object in jobs.values():
            if isinstance(job_object, dict):
                container = job_object.get("container", {})
                image = None
                start_line = 0
                end_line = 0

                if isinstance(container, dict):
                    image = container.get("image", "")
                    start_line = container.get('__startline__', 0)
                    end_line = container.get('__endline__', 0)

                elif isinstance(container, str):
                    image = container
                    start_line = [line_number for line_number, line in workflow_line_numbers if image in line][0]
                    end_line = start_line + 1

                if image:
                    image_obj = Image(
                        file_path=file_path,
                        name=image,
                        start_line=start_line,
                        end_line=end_line,
                        related_resource_id=Runner.generate_resource_key(workflow, start_line, end_line)
                    )
                    images.add(image_obj)

        return images

    @staticmethod
    def get_start_and_end_lines(key: str) -> list[int]:
        check_name = key.split('.')[-1]
        try:
            start_end_line_bracket_index = check_name.index('[')
        except ValueError:
            return [-1, -1]
        return [int(x) for x in check_name[start_end_line_bracket_index + 1: len(check_name) - 1].split(':')]

    @staticmethod
    def resolve_job_name(definition: dict[str, Any], start_line: int, end_line: int) -> str:
        for key, job in definition.get('jobs', {}).items():
            if key in [START_LINE, END_LINE]:
                continue
            if job[START_LINE] <= start_line <= end_line <= job[END_LINE]:
                return str(key)
        return ""

    @staticmethod
    def resolve_step_name(job_definition: dict[str, Any], start_line: int, end_line: int) -> str:
        for step in job_definition.get('steps', {}):
            if step[START_LINE] <= start_line <= end_line <= step[END_LINE]:
                try:
                    name = step["name"]
                except KeyError:
                    name = step[next(iter(step.keys()))]

                return cast(str, name)

        return ""

    @staticmethod
    def is_schema_valid(config: dict[str, Any] | list[dict[str, Any]]) -> bool:
        valid = False
        try:
            schema.validate(config)
            valid = True
        except SchemaError as e:
            logging.info(f'Given entity configuration does not match the schema\n'
                         f'config={json.dumps(config, indent=4)}\n'
                         f'schema={json.dumps(schema.json_schema("https://example.com/my-schema.json"), indent=4)}')
            logging.info(f'Error: {e}', exc_info=e)

        return valid




from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class ImageDigest(BaseResourceCheck):

    def __init__(self):
        """
         The image specification should use a digest instead of a tag to make sure the container always uses the same
         version of the image.
         https://kubernetes.io/docs/concepts/configuration/overview/#container-images

         An admission controller could be used to enforce the use of image digest
         """
        name = "Image should use digest"
        id = "CKV_K8S_43"
        supported_resources = ["kubernetes_pod"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        spec = conf.get('spec')[0]
        if spec:
            containers = spec.get("container")
            for idx, container in enumerate(containers):
                if not isinstance(container, dict):
                    return CheckResult.UNKNOWN
                if container.get("image") and isinstance(container.get("image"), list):
                    name = container.get("image")[0]
                    if "@" not in name:
                        self.evaluated_keys = [f'spec/[0]/container/[{idx}]/image']
                        return CheckResult.FAILED
            return CheckResult.PASSED
        return CheckResult.FAILED


check = ImageDigest()



from __future__ import annotations

import asyncio
import json
import logging
import os.path
from collections.abc import Iterable
from pathlib import Path
from typing import Optional, List, Union, Dict, Any

import requests

from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.bridgecrew.platform_key import bridgecrew_dir
from checkov.common.bridgecrew.vulnerability_scanning.image_scanner import image_scanner, TWISTCLI_FILE_NAME
from checkov.common.bridgecrew.vulnerability_scanning.integrations.docker_image_scanning import \
    docker_image_scanning_integration
from checkov.common.images.image_referencer import ImageReferencer, Image
from checkov.common.output.report import Report, merge_reports
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.output.common import ImageDetails
from checkov.common.runners.base_runner import filter_ignored_paths, strtobool
from checkov.common.sca.output import parse_vulns_to_records, get_license_statuses
from checkov.common.util.file_utils import compress_file_gzip_base64
from checkov.common.util.dockerfile import is_docker_file
from checkov.runner_filter import RunnerFilter
from checkov.sca_package.runner import Runner as PackageRunner


class Runner(PackageRunner):
    check_type = CheckType.SCA_IMAGE  # noqa: CCE003  # a static attribute

    def __init__(self) -> None:
        super().__init__(report_type=self.check_type)
        self._check_class: Optional[str] = None
        self._code_repo_path: Optional[Path] = None
        self._check_class = f"{image_scanner.__module__}.{image_scanner.__class__.__qualname__}"
        self.raw_report: Optional[Dict[str, Any]] = None
        self.base_url = bc_integration.api_url
        self.image_referencers: set[ImageReferencer] | None = None

    def should_scan_file(self, filename: str) -> bool:
        return is_docker_file(os.path.basename(filename))

    def scan(
            self,
            image_id: str,
            dockerfile_path: str,
            runner_filter: RunnerFilter | None = None,
    ) -> Dict[str, Any]:
        runner_filter = runner_filter or RunnerFilter()

        # skip complete run, if flag '--check' was used without a CVE check ID
        if runner_filter.checks and all(not check.startswith("CKV_CVE") for check in runner_filter.checks):
            return {}

        if not bc_integration.bc_api_key:
            logging.info("The --bc-api-key flag needs to be set to run SCA package scanning")
            return {}

        logging.info(f"SCA image scanning is scanning the image {image_id}")

        cached_results: Dict[str, Any] = image_scanner.get_scan_results_from_cache(image_id)
        if cached_results:
            logging.info(f"Found cached scan results of image {image_id}")
            return cached_results

        image_scanner.setup_scan(image_id, dockerfile_path, skip_extract_image_name=False)
        try:
            output_path = Path(f'results-{image_id}.json')
            scan_result = asyncio.run(self.execute_scan(image_id, output_path))
            self.upload_results_to_cache(output_path, image_id)
            logging.info(f"SCA image scanning successfully scanned the image {image_id}")
            return scan_result
        except Exception:
            raise

    async def execute_scan(
            self,
            image_id: str,
            output_path: Path,
    ) -> Dict[str, Any]:
        command = f"{Path(bridgecrew_dir) / TWISTCLI_FILE_NAME} images scan --address {docker_image_scanning_integration.get_proxy_address()} --token {docker_image_scanning_integration.get_bc_api_key()} --details --output-file \"{output_path}\" {image_id}"
        process = await asyncio.create_subprocess_shell(
            command, stdout=asyncio.subprocess.PIPE, stderr=asyncio.subprocess.PIPE
        )

        stdout, stderr = await process.communicate()

        # log output for debugging
        logging.debug(stdout.decode())

        exit_code = await process.wait()

        if exit_code:
            logging.error(stderr.decode())
            return {}

        # read the report file
        scan_result: Dict[str, Any] = json.loads(output_path.read_text())

        return scan_result

    def upload_results_to_cache(self, output_path: Path, image_id: str) -> None:
        image_id_sha = f"sha256:{image_id}" if not image_id.startswith("sha256:") else image_id

        request_body = {
            "compressedResult": compress_file_gzip_base64(str(output_path)),
            "compressionMethod": "gzip",
            "id": image_id_sha
        }
        response = requests.request(
            "POST", f"{self.base_url}/api/v1/vulnerabilities/scan-results",
            headers=bc_integration.get_default_headers("POST"), data=json.dumps(request_body)
        )

        if response.ok:
            logging.info(f"Successfully uploaded scan results to cache with id={image_id}")
        else:
            logging.info(f"Failed to upload scan results to cache with id={image_id}")

        output_path.unlink()

    def run(
            self,
            root_folder: Union[str, Path],
            external_checks_dir: Optional[List[str]] = None,
            files: Optional[List[str]] = None,
            runner_filter: RunnerFilter | None = None,
            collect_skip_comments: bool = True,
            **kwargs: str
    ) -> Report:
        runner_filter = runner_filter or RunnerFilter()
        if not runner_filter.show_progress_bar:
            self.pbar.turn_off_progress_bar()

        self._code_repo_path = Path(root_folder) if root_folder else None

        if "dockerfile_path" in kwargs and "image_id" in kwargs:
            dockerfile_path = kwargs['dockerfile_path']
            image_id = kwargs['image_id']
            return self.get_image_id_report(dockerfile_path, image_id, runner_filter)
        report = Report(self.check_type)
        if not files and not root_folder:
            logging.debug("No resources to scan.")
            return report
        if files:
            self.pbar.initiate(len(files))
            for file in files:
                self.pbar.set_additional_data({'Current File Scanned': os.path.relpath(file, root_folder)})
                self.iterate_image_files(file, report, runner_filter)
                self.pbar.update()
            self.pbar.close()
        if root_folder:
            for root, d_names, f_names in os.walk(root_folder):
                filter_ignored_paths(root, d_names, runner_filter.excluded_paths, included_paths=self.included_paths())
                filter_ignored_paths(root, f_names, runner_filter.excluded_paths, included_paths=self.included_paths())
                for file in f_names:
                    abs_fname = os.path.join(root, file)
                    self.iterate_image_files(abs_fname, report, runner_filter)
        return report

    def iterate_image_files(self, abs_fname: str, report: Report, runner_filter: RunnerFilter) -> None:
        """
        Get workflow file, and get the list of images from every relevant imagereferencer, and create a unified vulnrability report
        :param abs_fname: file path to inspect
        :param report: unified report object
        :param runner_filter: filter for report
        """
        if not self.image_referencers:
            return
        for image_referencer in self.image_referencers:
            if image_referencer.is_workflow_file(abs_fname):
                images = image_referencer.get_images(file_path=abs_fname)
                for image in images:
                    if not strtobool(os.getenv('CHECKOV_PRESENT_CACHED_RESULTS', "False")):
                        image_report = self.get_image_report(dockerfile_path=abs_fname, image=image,
                                                             runner_filter=runner_filter)
                        merge_reports(report, image_report)
                    else:
                        image_cached_report: dict[str, Any] = self.get_image_cached_results(dockerfile_path=abs_fname, image=image)
                        report.image_cached_results.append(image_cached_report)

    def get_report_from_scan_result(self, result: Dict[str, Any], dockerfile_path: str, rootless_file_path: str,
                                    image_details: ImageDetails | None, runner_filter: RunnerFilter) -> Report:
        report = Report(self.check_type)
        vulnerabilities = result.get("vulnerabilities", [])
        packages = result.get("packages", [])
        license_statuses = get_license_statuses(packages)

        parse_vulns_to_records(
            report=report,
            check_class=self._check_class,
            scanned_file_path=os.path.abspath(dockerfile_path),
            rootless_file_path=rootless_file_path,
            runner_filter=runner_filter,
            vulnerabilities=vulnerabilities,
            packages=packages,
            license_statuses=license_statuses,
            image_details=image_details,
            report_type=self.report_type,
        )
        return report

    def get_image_cached_results(self, dockerfile_path: str, image: Image) -> dict[str, Any]:
        """
            :param dockerfile_path: path of a file that might contain a container image
            :param image: Image object
            :return: cached_results report
        """
        cached_results: Dict[str, Any] = image_scanner.get_scan_results_from_cache(f"image:{image.name}")
        payload: dict[str, Any] = docker_image_scanning_integration.create_report(
            twistcli_scan_result=cached_results,
            bc_platform_integration=bc_integration,
            file_path=dockerfile_path,
            file_content=image.file_path,
            docker_image_name=image.name,
            related_resource_id=image.related_resource_id)
        return payload

    def get_image_report(self, dockerfile_path: str, image: Image, runner_filter: RunnerFilter) -> Report:
        """

        :param dockerfile_path: path of a file that might contain a container image
        :param image: Image object
        :param runner_filter:
        :return: vulnerability report
        """
        # skip complete run, if flag '--check' was used without a CVE check ID
        if runner_filter.checks and all(not check.startswith("CKV_CVE") for check in runner_filter.checks):
            return Report(self.check_type)

        cached_results: Dict[str, Any] = image_scanner.get_scan_results_from_cache(f"image:{image.name}")

        if cached_results:
            logging.info(f"Found cached scan results of image {image.name}")
            self.raw_report = cached_results
            result = cached_results.get('results', [{}])[0]
            image_id = self.extract_image_short_id(result)
            image_details = self.get_image_details_from_twistcli_result(scan_result=result, image_id=image_id,
                                                                        image_name=image.name,
                                                                        related_resource_id=image.related_resource_id)
            if self._code_repo_path:
                try:
                    dockerfile_path = str(Path(dockerfile_path).relative_to(self._code_repo_path))
                except ValueError:
                    # Path.is_relative_to() was implemented in Python 3.9
                    pass
            rootless_file_path = dockerfile_path.replace(Path(dockerfile_path).anchor, "", 1)
            rootless_file_path_to_report = f"{rootless_file_path} ({image.name} lines:{image.start_line}-" \
                                           f"{image.end_line} ({image_id}))"
            return self.get_report_from_scan_result(result, dockerfile_path, rootless_file_path_to_report,
                                                    image_details, runner_filter)

        elif strtobool(os.getenv("CHECKOV_EXPERIMENTAL_IMAGE_REFERENCING", "False")):
            # experimental flag on running image referencers via local twistcli
            image_id = ImageReferencer.inspect(image.name)
            scan_result = self.scan(image_id, dockerfile_path, runner_filter)
            if scan_result is None:
                return Report(self.check_type)

            self.raw_report = scan_result
            result = scan_result.get('results', [{}])[0]
            rootless_file_path_to_report = f"{dockerfile_path} ({image.name} lines:{image.start_line}-" \
                                           f"{image.end_line} ({image_id}))"
            return self.get_report_from_scan_result(result, dockerfile_path, rootless_file_path_to_report, None,
                                                    runner_filter)
        else:
            logging.info(f"No cache hit for image {image.name}")

        return Report(self.check_type)

    def get_image_id_report(self, dockerfile_path: str, image_id: str, runner_filter: RunnerFilter) -> Report:
        """
        THIS METHOD HANDLES CUSTOM IMAGE SCANNING THAT COMES DIRECTLY FROM CLI PARAMETERS
        """
        scan_result = self.scan(image_id, dockerfile_path, runner_filter)
        if scan_result is None:
            return Report(self.check_type)
        self.raw_report = scan_result
        result = scan_result.get('results', [{}])[0]
        image_details = self.get_image_details_from_twistcli_result(scan_result=result, image_id=image_id)
        if self._code_repo_path:
            try:
                dockerfile_path = str(Path(dockerfile_path).relative_to(self._code_repo_path))
            except ValueError:
                # Path.is_relative_to() was implemented in Python 3.9
                pass
        rootless_file_path = dockerfile_path.replace(Path(dockerfile_path).anchor, "", 1)
        rootless_file_path_to_report = f"{rootless_file_path} ({image_id})"
        return self.get_report_from_scan_result(result, dockerfile_path, rootless_file_path_to_report, image_details,
                                                runner_filter)

    def extract_image_short_id(self, scan_result: dict[str, Any]) -> str:
        """Extracts a shortened version of the image ID from the scan result"""

        if "id" not in scan_result:
            return "sha256:unknown"

        image_id: str = scan_result["id"]

        if image_id.startswith("sha256:"):
            return image_id[:17]
        return image_id[:10]

    def get_image_details_from_twistcli_result(self, scan_result: dict[str, Any], image_id: str,
                                               image_name: str | None = None,
                                               related_resource_id: str | None = None) -> ImageDetails:
        image_packages = scan_result.get('packages', [])
        image_package_types = {
            f'{package["name"]}@{package["version"]}': package['type']
            for package in image_packages
        }
        return ImageDetails(
            distro=scan_result.get('distro', ''),
            distro_release=scan_result.get('distroRelease', ''),
            package_types=image_package_types,
            image_id=image_id,
            name=image_name,
            related_resource_id=related_resource_id
        )

    def included_paths(self) -> Iterable[str]:
        return ['.github', '.circleci']



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class AllowedCapabilitiesSysAdmin(BaseResourceCheck):

    def __init__(self):
        name = "Do not use the CAP_SYS_ADMIN linux capability"
        # This provides the most privilege and is similar to root
        # https://kubernetes.io/docs/tasks/configure-pod-container/security-context/
        id = "CKV_K8S_39"

        supported_resources = ['kubernetes_pod']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        spec = conf.get('spec', [None])[0]
        if isinstance(spec, dict) and spec.get("container"):
            containers = spec.get("container")

            for idx, container in enumerate(containers):
                if type(container) != dict:
                    return CheckResult.UNKNOWN
                if container.get("security_context") and isinstance(container.get("security_context"), list):
                    context = container.get("security_context")[0]
                    if context.get("capabilities") and isinstance(context.get("capabilities"), list):
                        capabilities = context.get("capabilities")[0]
                        if capabilities.get("add") and isinstance(capabilities.get("add"), list):
                            add = capabilities.get("add")[0]
                            if "SYS_ADMIN" in add:
                                self.evaluated_keys = [f'spec/[0]/container/[{idx}]/'
                                                       f'security_context/[0]/capabilities/add']
                                return CheckResult.FAILED
        return CheckResult.PASSED


check = AllowedCapabilitiesSysAdmin()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from typing import Any


class K8sNodePoolAutoRepair(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure K8s nodepools are set to auto repair"
        id = "CKV_ALI_31"
        supported_resources = ['alicloud_cs_kubernetes_node_pool']
        categories = [CheckCategories.KUBERNETES]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.FAILED)

    def get_inspected_key(self):
        return "management/auto_repair"


check = K8sNodePoolAutoRepair()



from checkov.terraform.checks.resource.aws.AbsNACLUnrestrictedIngress import AbsNACLUnrestrictedIngress


class NACLUnrestrictedIngress20(AbsNACLUnrestrictedIngress):
    def __init__(self):
        super().__init__(check_id="CKV_AWS_230", port=20)


check = NACLUnrestrictedIngress20()



from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AthenaWorkgroupEncryption(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure that Athena Workgroup is encrypted"
        id = "CKV_AWS_159"
        supported_resources = ['aws_athena_workgroup']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "configuration/[0]/result_configuration/[0]/encryption_configuration/[0]/encryption_option"

    def get_expected_value(self):
        return ANY_VALUE

check = AthenaWorkgroupEncryption()



import os

from checkov.common.runners.base_runner import strtobool
from checkov.common.vcs.base_vcs_dal import BaseVCSDAL


class Gitlab(BaseVCSDAL):
    def __init__(self):
        super().__init__()

    def setup_conf_dir(self):
        """
            discover parameters from execution context of checkov and determine the directory to save temporal files of vcs configuration
        """
        gitlab_conf_dir_name = os.getenv('CKV_GITLAB_CONF_DIR_NAME', 'gitlab_conf')
        self.gitlab_conf_dir_path = os.path.join(os.getcwd(), gitlab_conf_dir_name)
        self.gitlab_project_approvals_file_path = os.path.join(self.gitlab_conf_dir_path,
                                                               "project_approvals.json")
        self.gitlab_groups_file_path = os.path.join(self.gitlab_conf_dir_path,
                                                    "groups.json")

    def discover(self):
        """
            discover parameters from execution context of checkov. usually from env variable
        """
        server_host = os.getenv('CI_SERVER_URL', "https://gitlab.com")
        self.api_url = f'{server_host}/api/v4/'
        self.graphql_api_url = f"{server_host}/api/graphql"

        self.token = os.getenv('CI_JOB_TOKEN', '')

        self.current_repository = os.getenv('CI_MERGE_REQUEST_PROJECT_PATH', '')
        self.current_branch = os.getenv('CI_COMMIT_REF_NAME', '')
        self.group_name = os.getenv('CI_PROJECT_NAMESPACE', '')
        self.project_id = os.getenv('CI_PROJECT_ID', '')
        self.default_branch_cache = {}

    def _headers(self):
        return {"Authorization": "Bearer {}".format(self.token)}

    def get_project_approvals(self):
        if self.project_id:
            project_approvals = self._request(endpoint=f"projects/{self.project_id}/approvals",
                                              allowed_status_codes=[200])
            return project_approvals
        return None

    def persist_project_approvals(self):
        project_approvals = self.get_project_approvals()

        if project_approvals:
            BaseVCSDAL.persist(path=self.gitlab_project_approvals_file_path, conf=project_approvals)

    def get_groups(self):
        groups = self._request(endpoint="groups", allowed_status_codes=[200])
        return groups

    def persist_groups(self):
        groups = self.get_groups()
        if groups:
            BaseVCSDAL.persist(path=self.gitlab_groups_file_path, conf=groups)

    def persist_all_confs(self):
        if strtobool(os.getenv("CKV_GITLAB_CONFIG_FETCH_DATA", "True")):
            self.persist_project_approvals()
            self.persist_groups()



from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class KinesisFirehoseDeliveryStreamSSE(BaseResourceCheck):
    def __init__(self) -> None:
        """
        Server-side encryption should not be enabled when a kinesis stream is configured
        as the source of the firehose delivery stream.
        """
        name = "Ensure Kinesis Firehose delivery stream is encrypted"
        id = "CKV_AWS_240"
        supported_resources = ["aws_kinesis_firehose_delivery_stream"]
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        if conf.get("kinesis_source_configuration"):
            return CheckResult.UNKNOWN
        if conf.get("server_side_encryption"):
            sse = conf.get("server_side_encryption")[0]
            if sse.get("enabled") == [True]:
                return CheckResult.PASSED
        self.evaluated_keys = ["server_side_encryption/[0]/enabled"]
        return CheckResult.FAILED


check = KinesisFirehoseDeliveryStreamSSE()



from abc import abstractmethod
from collections.abc import Iterable
from typing import Dict, Any, Optional

from checkov.common.checks.base_check import BaseCheck
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.common.multi_signature import multi_signature
from checkov.kubernetes.checks.resource.registry import registry


class BaseK8Check(BaseCheck):
    def __init__(
        self,
        name: str,
        id: str,
        categories: "Iterable[CheckCategories]",
        supported_entities: "Iterable[str]",
        guideline: Optional[str] = None,
    ) -> None:
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_entities,
            block_type="k8",
            guideline=guideline
        )
        self.supported_specs = supported_entities
        registry.register(self)

    def scan_entity_conf(self, conf: Dict[str, Any], entity_type: str) -> CheckResult:
        self.entity_type = entity_type
        return self.scan_spec_conf(conf, entity_type)

    @multi_signature()
    @abstractmethod
    def scan_spec_conf(self, conf: Dict[str, Any], entity_type: str) -> CheckResult:
        """Return result of Kubernetes object check."""
        raise NotImplementedError()

    @classmethod
    @scan_spec_conf.add_signature(args=["self", "conf"])
    def _scan_spec_conf_self_conf(cls, wrapped):
        def wrapper(self, conf, entity_type=None):
            # keep default argument for entity_type so old code, that doesn't set it, will work.
            return wrapped(self, conf)

        return wrapper

    @staticmethod
    def get_inner_entry(conf: Dict[str, Any], entry_name: str) -> Dict[str, Any]:
        spec = {}
        if conf.get("spec") and conf.get("spec").get("template"):
            spec = conf.get("spec").get("template").get(entry_name, {})
        return spec



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class AbsRDSParameter(BaseResourceCheck):
    def __init__(self, check_id, parameter):
        name = f"Ensure RDS instance has {parameter} enabled"
        supported_resources = ['alicloud_db_instance']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=check_id, categories=categories, supported_resources=supported_resources)
        self.parameter = parameter

    def scan_resource_conf(self, conf):
        """
            Looks at configuration of RDS logging parameters :

        :param conf: alicloud_db_instance configuration
        :return: <CheckResult>
        """
        params = conf.get("parameters")
        if params and isinstance(params, list):
            for param in params:
                if param['name'][0] == self.parameter and param['value'][0] == 'ON':
                    return CheckResult.PASSED
        return CheckResult.FAILED



from __future__ import annotations
from checkov.common.output.common import ImageDetails

UNFIXABLE_VERSION = "N/A"

TWISTCLI_TO_CHECKOV_LANG_NORMALIZATION = {
    "gem": "ruby"
}

def get_file_path_for_record(rootless_file_path: str) -> str:
    return f"/{rootless_file_path}"


def get_resource_for_record(rootless_file_path: str, package_name: str) -> str:
    return f"{rootless_file_path}.{package_name}"


def get_package_alias(package_name: str, package_version: str) -> str:
    return f"{package_name}@{package_version}"


def get_package_type(package_name: str, package_version: str, image_details: ImageDetails | None = None) -> str:
    if image_details:
        return str(image_details.package_types.get(f"{package_name}@{package_version}", ""))
    else:
        return ""


def normalize_twistcli_language(language: str) -> str:
    """
    part of the language names that are returned by twistcli may be a little differ from those we use in checkov.
    this function's goal is to normalize them
    """
    return TWISTCLI_TO_CHECKOV_LANG_NORMALIZATION.get(language, language)



from typing import Any, Dict

from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_container_check import BaseK8sContainerCheck


class Tiller(BaseK8sContainerCheck):
    def __init__(self) -> None:
        name = "Ensure that Tiller (Helm v2) is not deployed"
        id = "CKV_K8S_34"
        # Location: container .image
        super().__init__(name=name, id=id)

    def scan_container_conf(self, metadata: Dict[str, Any], conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_container_keys = ["image"]
        return CheckResult.FAILED if Tiller.is_tiller(metadata, conf) else CheckResult.PASSED

    @staticmethod
    def is_tiller(metadata: Dict[str, Any], conf: Dict[str, Any]) -> bool:
        image = conf.get("image")
        if image and isinstance(image, str) and "tiller" in image:
            return True

        if metadata:
            labels = metadata.get("labels")
            if labels:
                if labels.get("app") == "helm":
                    return True
                elif labels.get("name") == "tiller":
                    return True

        return False


check = Tiller()



import logging
from typing import Dict, TYPE_CHECKING, Tuple, List, Any

import dpath.util

if TYPE_CHECKING:
    from checkov.terraform.context_parsers.base_parser import BaseContextParser


class ParserRegistry:
    context_parsers: Dict[str, "BaseContextParser"] = {}
    definitions_context: Dict[str, Dict[str, Dict[str, Any]]] = {}

    def __init__(self) -> None:
        self.logger = logging.getLogger(__name__)

    def register(self, parser: "BaseContextParser") -> None:
        self.context_parsers[parser.definition_type] = parser

    def reset_definitions_context(self) -> None:
        self.definitions_context = {}

    def enrich_definitions_context(
        self, definitions: Tuple[str, Dict[str, List[Dict[str, Any]]]], collect_skip_comments: bool = True
    ) -> Dict[str, Dict[str, Dict[str, Any]]]:
        supported_definitions = [parser_type for parser_type in self.context_parsers.keys()]
        (tf_file, definition_blocks_types) = definitions
        if definition_blocks_types:
            definition_blocks_types = {x: definition_blocks_types[x] for x in definition_blocks_types.keys()}
            for definition_type in definition_blocks_types.keys():
                if definition_type in supported_definitions:
                    dpath.new(self.definitions_context, [tf_file, definition_type], {})
                    context_parser = self.context_parsers[definition_type]
                    definition_blocks = definition_blocks_types[definition_type]
                    self.definitions_context[tf_file][definition_type] = context_parser.run(
                        tf_file, definition_blocks, collect_skip_comments
                    )

        return self.definitions_context


parser_registry = ParserRegistry()



from __future__ import annotations

from typing import Any

from checkov.bicep.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class StorageAccountAzureServicesAccessEnabled(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure 'Trusted Microsoft Services' is enabled for Storage Account access"
        id = "CKV_AZURE_36"
        supported_resources = ("Microsoft.Storage/storageAccounts",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        self.evaluated_keys = ["properties/networkAcls/defaultAction"]
        properties = conf.get("properties")
        if properties:
            nacls = properties.get("networkAcls")
            if nacls:
                default_action = nacls.get("defaultAction")
                if default_action == "Deny":
                    bypass = nacls.get("bypass")
                    if not bypass or bypass == "None":
                        self.evaluated_keys.append("properties/networkAcls/bypass")
                        return CheckResult.FAILED

        return CheckResult.PASSED


check = StorageAccountAzureServicesAccessEnabled()



import dpath

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AKSLoggingEnabled(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure AKS logging to Azure Monitoring is Configured"
        id = "CKV_AZURE_4"
        supported_resources = ['azurerm_kubernetes_cluster']
        categories = [CheckCategories.KUBERNETES]
        self.provider_version_2_path = "addon_profile/[0]/oms_agent/[0]/enabled"
        self.provider_version_3_path = "oms_agent/[0]/log_analytics_workspace_id"
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if dpath.search(conf, self.provider_version_2_path):
            self.evaluated_keys = [self.provider_version_2_path]
            return super().scan_resource_conf(conf)
        elif dpath.search(conf, self.provider_version_3_path):
            self.evaluated_keys = [self.provider_version_3_path]
            return CheckResult.PASSED

        return CheckResult.FAILED

    def get_inspected_key(self) -> str:
        return self.provider_version_2_path


check = AKSLoggingEnabled()



import os

from checkov.common.runners.base_runner import strtobool
from checkov.common.vcs.base_vcs_dal import BaseVCSDAL
from checkov.github.schemas.org_security import schema as org_security_schema


class Github(BaseVCSDAL):
    def __init__(self):
        super().__init__()

    def setup_conf_dir(self):
        # files downloaded from github will be persistent in this directory
        github_conf_dir_name = os.getenv('CKV_GITHUB_CONF_DIR_NAME', 'github_conf')
        self.github_conf_dir_path = os.path.join(os.getcwd(), github_conf_dir_name)
        self.github_org_security_file_path = os.path.join(self.github_conf_dir_path, "org_security.json")
        self.github_branch_protection_rules_file_path = os.path.join(self.github_conf_dir_path,
                                                                     "branch_protection_rules.json")
        self.github_org_webhooks_file_path = os.path.join(self.github_conf_dir_path,
                                                          "org_webhooks.json")
        self.github_repository_webhooks_file_path = os.path.join(self.github_conf_dir_path,
                                                                 "repository_webhooks.json")
        self.github_repository_collaborators_file_path = os.path.join(self.github_conf_dir_path,
                                                                      "repository_collaborators.json")

    def discover(self):

        self.api_url = os.getenv('GITHUB_API_URL', "https://api.github.com")
        self.graphql_api_url = f"{self.api_url}/graphql"

        self.token = os.getenv('GITHUB_TOKEN', '')

        self.current_repository = os.getenv('GITHUB_REPOSITORY', '')
        self.current_branch = os.getenv('GITHUB_REF_NAME', '')
        if not self.current_branch:
            self.current_branch = os.getenv('GITHUB_REF', 'refs/heads/master')
            if self.current_branch:
                extracted_branch_array = self.current_branch.split("/")
                if len(extracted_branch_array) == 3:
                    self.current_branch = extracted_branch_array[2]

        self.default_branch_cache = {}
        self.org = os.getenv('GITHUB_ORG', '')

    def _headers(self):
        return {"Accept": "application/vnd.github.v3+json",
                "Authorization": "token {}".format(self.token)}

    def get_branch_protection_rules(self):
        if self.current_branch and self.current_repository:
            branch_protection_rules = self._request(
                endpoint="repos/{}/branches/{}/protection".format(self.current_repository, self.current_branch),
                allowed_status_codes=[200, 404])
            return branch_protection_rules
        return None

    def persist_branch_protection_rules(self):
        data = self.get_branch_protection_rules()
        if data:
            BaseVCSDAL.persist(path=self.github_branch_protection_rules_file_path, conf=data)

    def persist_organization_security(self):
        organization_security = self.get_organization_security()
        if organization_security:
            BaseVCSDAL.persist(path=self.github_org_security_file_path, conf=organization_security)

    def persist_organization_webhooks(self):
        organization_webhooks = self.get_organization_webhooks()
        if organization_webhooks:
            BaseVCSDAL.persist(path=self.github_org_webhooks_file_path, conf=organization_webhooks)

    def get_organization_webhooks(self):
        data = self._request(endpoint="orgs/{}/hooks".format(self.org), allowed_status_codes=[200])
        if not data:
            return None
        return data

    def get_repository_collaborators(self):
        data = self._request(endpoint="repos/{}/{}/collaborators".format(self.org, self.current_repository),
                             allowed_status_codes=[200])
        if not data:
            return None
        return data

    def persist_repository_collaborators(self):
        repository_collaborators = self.get_repository_collaborators()
        if repository_collaborators:
            BaseVCSDAL.persist(path=self.github_repository_collaborators_file_path, conf=repository_collaborators)

    def get_repository_webhooks(self):
        data = self._request(endpoint="repos/{}/{}/hooks".format(self.org, self.current_repository),
                             allowed_status_codes=[200])
        if not data:
            return None
        return data

    def persist_repository_webhooks(self):
        repository_webhooks = self.get_repository_webhooks()
        if repository_webhooks:
            BaseVCSDAL.persist(path=self.github_repository_webhooks_file_path, conf=repository_webhooks)

    def get_organization_security(self):
        if not self._organization_security:
            data = self._request_graphql(query="""
                query ($org: String! ) {
                    organization(login: $org) {
                        name
                        login
                        description
                        ipAllowListEnabledSetting
                        ipAllowListForInstalledAppsEnabledSetting
                        requiresTwoFactorAuthentication
                        samlIdentityProvider {
                            ssoUrl
                        }
                    }
                }
                """, variables={'org': self.org})
            if not data:
                return None
            if org_security_schema.validate(data):
                self._organization_security = data
        return self._organization_security

    def persist_all_confs(self):
        if strtobool(os.getenv("CKV_GITHUB_CONFIG_FETCH_DATA", "True")):
            self.persist_organization_security()
            self.persist_branch_protection_rules()
            self.persist_organization_webhooks()
            self.persist_repository_webhooks()
            self.persist_repository_collaborators()



from __future__ import annotations

import jmespath
import os
from typing import TYPE_CHECKING, Any

from checkov.common.images.image_referencer import ImageReferencer, Image
from checkov.common.output.report import CheckType
from checkov.circleci_pipelines.registry import registry
from checkov.yaml_doc.runner import Runner as YamlRunner

if TYPE_CHECKING:
    from checkov.common.checks.base_check_registry import BaseCheckRegistry

WORKFLOW_DIRECTORY = "circleci"

class Runner(YamlRunner, ImageReferencer):
    check_type = CheckType.CIRCLECI_PIPELINES  # noqa: CCE003  # a static attribute

    def require_external_checks(self) -> bool:
        return False

    def import_registry(self) -> BaseCheckRegistry:
        return registry

    def included_paths(self) -> list[str]:
        return [".circleci"]

    def _parse_file(
        self, f: str, file_content: str | None = None
    ) -> tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
        if self.is_workflow_file(f):
            return super()._parse_file(f)

        return None

    def is_workflow_file(self, file_path: str) -> bool:
        """
        :return: True if the file mentioned is named config.yml/yaml in .circleci dir from included_paths(). Otherwise: False
        """
        abspath = os.path.abspath(file_path)
        return WORKFLOW_DIRECTORY in abspath and abspath.endswith(("config.yml", "config.yaml"))

    def get_images(self, file_path: str) -> set[Image]:
        """
        Get container images mentioned in a file
        :param file_path: File to be inspected

        File sample that will return 5 Image objects:
            # # Use the latest 2.1 version of CircleCI pipeline process engine.
            # # See: https://circleci.com/docs/2.0/configuration-reference
            # version: 2.1
            # # Define a job to be invoked later in a workflow.
            # # See: https://circleci.com/docs/2.0/configuration-reference/#jobs
            # jobs:
            # say-hello:
            #    docker:
            #    - image: buildpack-deps:latest # primary container
            #        auth:
            #        username: mydockerhub-user
            #        password: $DOCKERHUB_PASSWORD  # context / project UI env-var reference
            #        environment:
            #        ENV: CI
            #    - image: mongo:2.6.8
            #        auth:
            #        username: mydockerhub-user
            #        password: $DOCKERHUB_PASSWORD  # context / project UI env-var reference
            #        command: [--smallfiles]
            #    - image: postgres:14.2
            #        auth:
            #        username: mydockerhub-user
            #        password: $DOCKERHUB_PASSWORD  # context / project UI env-var reference
            #        environment:
            #        POSTGRES_USER: user
            #    - image: redis@sha256:54057dd7e125ca41afe526a877e8bd35ec2cdd33b9217e022ed37bdcf7d09673
            #        auth:
            #        username: mydockerhub-user
            #        password: $DOCKERHUB_PASSWORD  # context / project UI env-var reference
            #    - image: acme-private/private-image:321
            #        auth:
            #        username: mydockerhub-user
            #        password: $DOCKERHUB_PASSWORD  # context / project UI env-var reference
            # # Invoke jobs via workflows
            # # See: https://circleci.com/docs/2.0/configuration-reference/#workflows
            # workflows:
            # say-hello-workflow:
            #     jobs:
            #     - say-hello
        :return: List of container image objects mentioned in the file.

        """

        images: set[Image] = set()
        parsed_file = self._parse_file(file_path)

        if not parsed_file:
            return images

        workflow, workflow_line_numbers = parsed_file

        if not isinstance(workflow, dict):
            # make type checking happy
            return images

        images.union(self.add_docker_job_images(workflow, images, file_path))

        return images

    def add_docker_job_images(self, workflow: dict[str, Any], images: set[Image], file_path: str) -> set[Image]:
        """

        :param workflow: parsed workflow file
        :param images: set of images to be updated
        :param file_path: path of analyzed workflow
        """

        # Only 'docker' executor paths are supported. (machine, windows & osx executors do not yeild OCI compatible images see https://circleci.com/docs/2.0/configuration-reference#docker-machine-macos-windows-executor)
        keywords = [
            'jobs.*.docker[].{image: image, __startline__: __startline__, __endline__:__endline__}']
        for keyword in keywords:
            results = jmespath.search(keyword, workflow)
            for result in results:
                image_name = result.get("image", None)
                if image_name:
                    image_obj = Image(
                        file_path=file_path,
                        name=image_name,
                        start_line=result["__startline__"],
                        end_line=result["__endline__"],
                    )
                    images.add(image_obj)
        return images


from __future__ import annotations

import concurrent.futures
import re
from typing import List, Tuple, Dict, Any, Optional, Pattern, TYPE_CHECKING

from jsonpath_ng.ext import parse

from checkov.common.graph.checks_infra.enums import SolverType
from checkov.common.graph.checks_infra.solvers.base_solver import BaseSolver

from concurrent.futures import ThreadPoolExecutor

from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.graph.graph_builder.graph_components.block_types import BlockType
from checkov.common.util.var_utils import is_terraform_variable_dependent
from checkov.terraform.graph_builder.graph_components.block_types import BlockType as TerraformBlockType

if TYPE_CHECKING:
    from networkx import DiGraph

SUPPORTED_BLOCK_TYPES = {BlockType.RESOURCE, TerraformBlockType.DATA}
WILDCARD_PATTERN = re.compile(r"(\S+[.][*][.]*)+")

OPERATION_TO_FUNC = {
    'all': all,
    'any': any
}


class BaseAttributeSolver(BaseSolver):
    operator = ""  # noqa: CCE003  # a static attribute

    def __init__(self, resource_types: List[str], attribute: Optional[str], value: Any, is_jsonpath_check: bool = False
                 ) -> None:
        super().__init__(SolverType.ATTRIBUTE)
        self.resource_types = resource_types
        self.attribute = attribute
        self.value = value
        self.is_jsonpath_check = is_jsonpath_check

    def run(self, graph_connector: DiGraph) -> Tuple[List[Dict[str, Any]], List[Dict[str, Any]]]:
        executer = ThreadPoolExecutor()
        jobs = []
        passed_vertices: List[Dict[str, Any]] = []
        failed_vertices: List[Dict[str, Any]] = []
        for _, data in graph_connector.nodes(data=True):
            if (not self.resource_types or data.get(CustomAttributes.RESOURCE_TYPE) in self.resource_types) \
                    and data.get(CustomAttributes.BLOCK_TYPE) in SUPPORTED_BLOCK_TYPES:
                jobs.append(executer.submit(self._process_node, data, passed_vertices, failed_vertices))

        concurrent.futures.wait(jobs)
        return passed_vertices, failed_vertices

    def get_operation(self, vertex: Dict[str, Any]) -> bool:  # type:ignore[override]
        if self.attribute and (self.is_jsonpath_check or re.match(WILDCARD_PATTERN, self.attribute)):
            attribute_matches = self.get_attribute_matches(vertex)

            operator = OPERATION_TO_FUNC['all'] if self.is_jsonpath_check else OPERATION_TO_FUNC['any']
            if attribute_matches:
                return self.resource_type_pred(vertex, self.resource_types) and operator(
                    self._get_operation(vertex=vertex, attribute=attr) for attr in attribute_matches
                )

        return self.resource_type_pred(vertex, self.resource_types) and self._get_operation(
            vertex=vertex, attribute=self.attribute
        )

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:  # type:ignore[override]
        raise NotImplementedError

    def _process_node(
        self, data: Dict[str, Any], passed_vartices: List[Dict[str, Any]], failed_vertices: List[Dict[str, Any]]
    ) -> None:
        if not self.resource_type_pred(data, self.resource_types):
            return
        if self.get_operation(vertex=data):
            passed_vartices.append(data)
        else:
            failed_vertices.append(data)

    def get_attribute_matches(self, vertex: Dict[str, Any]) -> List[str]:
        attribute_matches: List[str] = []
        if self.is_jsonpath_check:
            parsed_attr = parse(self.attribute)
            for match in parsed_attr.find(vertex):
                full_path = str(match.full_path)
                if full_path not in vertex:
                    vertex[full_path] = match.value

                attribute_matches.append(full_path)

        elif isinstance(self.attribute, str):
            attribute_patterns = self.get_attribute_patterns(self.attribute)
            for attr in vertex:
                if any(re.match(re.compile(attribute_pattern), attr) for attribute_pattern in attribute_patterns):
                    attribute_matches.append(attr)

        return attribute_matches

    @staticmethod
    def get_attribute_patterns(attribute: str) -> Tuple[Pattern[str], Pattern[str]]:
        index_pattern = r"[\d]+"
        split_by_dots = attribute.split(".")

        pattern_parts = []
        pattern_parts_without_index = []
        for attr_part in split_by_dots:
            if attr_part == "*":
                pattern_parts.append(index_pattern)
            else:
                attr_part_pattern = f"({attr_part})"
                pattern_parts.append(attr_part_pattern)
                pattern_parts_without_index.append(attr_part_pattern)

        pattern = f'^{"[.]".join(pattern_parts)}$'
        pattern_with_index = re.compile(pattern)

        pattern = f'^{"[.]".join(pattern_parts_without_index)}$'
        pattern_without_index = re.compile(pattern)

        return pattern_with_index, pattern_without_index

    @staticmethod
    def _is_variable_dependant(value: Any, source: str) -> bool:
        if source == 'Terraform' and is_terraform_variable_dependent(value):
            return True
        # TODO add logic for CloudFormation
        # elif source == 'CloudFormation' and is_cloudformation_variable_dependent(value):
        #     return True

        return False



from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from checkov.common.bridgecrew.integration_features.base_integration_feature import BaseIntegrationFeature
    from checkov.common.output.report import Report


class IntegrationFeatureRegistry:
    def __init__(self) -> None:
        self.features: list[BaseIntegrationFeature] = []

    def register(self, integration_feature: BaseIntegrationFeature) -> None:
        self.features.append(integration_feature)
        self.features.sort(key=lambda f: f.order)

    def run_pre_scan(self) -> None:
        for integration in self.features:
            if integration.is_valid():
                integration.pre_scan()

    def run_pre_runner(self) -> None:
        for integration in self.features:
            if integration.is_valid():
                integration.pre_runner()

    def run_post_runner(self, scan_report: Report) -> None:
        for integration in self.features:
            if integration.is_valid():
                integration.post_runner(scan_report)


integration_feature_registry = IntegrationFeatureRegistry()



from __future__ import annotations

from typing import List, Optional, Dict, Any, Tuple, TYPE_CHECKING

from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.checks_infra.solvers.connections_solvers.connection_exists_solver import ConnectionExistsSolver

if TYPE_CHECKING:
    from networkx import DiGraph


class ConnectionNotExistsSolver(ConnectionExistsSolver):
    operator = Operators.NOT_EXISTS  # noqa: CCE003  # a static attribute

    def __init__(
        self,
        resource_types: List[str],
        connected_resources_types: List[str],
        vertices_under_resource_types: Optional[List[Dict[str, Any]]] = None,
        vertices_under_connected_resources_types: Optional[List[Dict[str, Any]]] = None,
    ) -> None:
        super().__init__(
            resource_types,
            connected_resources_types,
            vertices_under_resource_types,
            vertices_under_connected_resources_types,
        )

    def get_operation(self, graph_connector: DiGraph) -> Tuple[List[Dict[str, Any]], List[Dict[str, Any]]]:  # type:ignore[override]
        passed, failed = super().get_operation(graph_connector)
        return failed, passed



from typing import List, Any, Dict

from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.graph.checks_infra.solvers.base_solver import BaseSolver
from checkov.common.checks_infra.solvers.complex_solvers.base_complex_solver import BaseComplexSolver


class NotSolver(BaseComplexSolver):
    operator = Operators.NOT  # noqa: CCE003  # a static attribute

    def __init__(self, solvers: List[BaseSolver], resource_types: List[str]) -> None:
        if len(solvers) != 1:
            raise Exception('The "not" operator must have exactly one child')
        super().__init__(solvers, resource_types)

    def _get_operation(self, *args: Any, **kwargs: Any) -> Any:
        if len(args) != 1:
            raise Exception('The "not" operator must have exactly one child')
        return not args[0]

    def get_operation(self, vertex: Dict[str, Any]) -> bool:  # type:ignore[override]
        return not self.solvers[0].get_operation(vertex)



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class AllowPrivilegeEscalation(BaseResourceCheck):
    def __init__(self):
        # CIS-1.3 1.7.5
        # CIS-1.5 5.2.5
        # https://kubernetes.io/docs/tasks/configure-pod-container/security-context/
        # AllowPrivilegeEscalation is true always when the container is: 1) run as Privileged OR 2) has CAP_SYS_ADMIN.
        # https://kubernetes.io/docs/concepts/policy/pod-security-policy/
        # AllowPrivilegeEscalation - This defaults to allow to not break setuid binaries
        # DefaultAllowPrivilegeEscalation - Default is to allow as to not break setuid binaries

        name = "Containers should not run with allowPrivilegeEscalation"
        id = "CKV_K8S_20"
        supported_resources = ['kubernetes_pod']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        spec_list = conf.get('spec')
        if not spec_list:
            return CheckResult.UNKNOWN
        spec = spec_list[0]
        if spec:
            containers = spec.get("container")
            if not containers:
                return CheckResult.UNKNOWN
            for idx, container in enumerate(containers):
                if not isinstance(container, dict):
                    return CheckResult.UNKNOWN
                if container.get("security_context"):
                    context = container.get("security_context")[0]
                    if context.get("allow_privilege_escalation"):
                        if context.get("allow_privilege_escalation") == [True]:
                            self.evaluated_keys = [f'spec/[0]/container/[{idx}]/security_context/[0]/'
                                                   f'allow_privilege_escalation']
                            return CheckResult.FAILED
            return CheckResult.PASSED
        return CheckResult.UNKNOWN


check = AllowPrivilegeEscalation()



from __future__ import annotations

import logging
from typing import Any

from yaml.parser import ScannerError
from yaml import YAMLError

from checkov.arm.parser import cfn_yaml
from checkov.common.parsers.json import parse as json_parse
from checkov.common.parsers.node import DictNode


LOGGER = logging.getLogger(__name__)


def parse(filename: str) -> tuple[dict[str, Any], tuple[int, str]] | tuple[None, None]:
    """ Decode filename into an object """

    template = None
    template_lines = None
    try:
        (template, template_lines) = cfn_yaml.load(filename)
    except IOError as e:
        if e.errno == 2:
            LOGGER.error(f"Template file not found: {filename}")
        elif e.errno == 21:
            LOGGER.error(f"Template references a directory, not a file: {filename}")
        elif e.errno == 13:
            LOGGER.error(f"Permission denied when accessing template file: {filename}")
    except UnicodeDecodeError:
        LOGGER.error(f"Cannot read file contents: {filename}")
    except cfn_yaml.CfnParseError:
        pass
    except ScannerError as err:
        if err.problem in ("found character '\\t' that cannot start any token", "found unknown escape character"):
            try:
                (template, template_lines) = json_parse(filename, allow_nulls=False)
            except Exception:
                LOGGER.error(f"Template {filename} is malformed: {err.problem}")
                LOGGER.error(f"Tried to parse {filename} as JSON", exc_info=True)
    except YAMLError:
        pass

    if template is not None and isinstance(template, DictNode) and "$schema" in template and "resources" in template:
        return template, template_lines
    return None, None



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from typing import List, Any


class RDSInstanceAutoUpgradeMinorVersion(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure RDS instance is set to auto upgrade minor versions"
        id = "CKV_ALI_34"
        supported_resources = ['alicloud_db_instance']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'auto_upgrade_minor_version'

    def get_expected_value(self) -> Any:
        return "Auto"


check = RDSInstanceAutoUpgradeMinorVersion()



from __future__ import annotations
from typing import Any

from checkov.circleci_pipelines.base_circleci_pipelines_check import BaseCircleCIPipelinesCheck
from checkov.common.models.enums import CheckResult
from checkov.yaml_doc.enums import BlockType

class PreventDevelopmentOrbs(BaseCircleCIPipelinesCheck):
    def __init__(self) -> None:
        name = "Ensure mutable development orbs are not used."
        id = "CKV_CIRCLECIPIPELINES_3"
        super().__init__(
            name=name,
            id=id,
            block_type=BlockType.ARRAY,
            supported_entities=["orbs.{orbs: @}"]
        )

    def scan_conf(self, conf: dict[str, Any]) -> tuple[CheckResult, dict[str, Any]]:
        for orb in conf.values():
            if isinstance(orb, str):
                # Special __ vars show up in this dict too.
                if "@dev" in orb:
                    # We only get one return per orb: section, regardless of how many orbs, so set a flag and error later.
                    # Potentially more JMEpath reflection-foo can resolve this so we end up with a call to scan_entity_conf per orb.
                    return CheckResult.FAILED, conf
        
        return CheckResult.PASSED, conf

check = PreventDevelopmentOrbs()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.dockerfile.base_dockerfile_check import BaseDockerfileCheck


class UserExists(BaseDockerfileCheck):
    def __init__(self) -> None:
        name = "Ensure that a user for the container has been created"
        id = "CKV_DOCKER_3"
        supported_instructions = ("*",)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_instructions=supported_instructions)

    def scan_entity_conf(self, conf: dict[str, Any]) -> tuple[CheckResult, dict[str, Any] | None]:
        for instruction, content in conf.items():
            if instruction == "USER":
                return CheckResult.PASSED, content[0]
        return CheckResult.FAILED, None


check = UserExists()



from typing import Optional, Any, Dict

from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.checks_infra.solvers.attribute_solvers.base_attribute_solver import BaseAttributeSolver


class ExistsAttributeSolver(BaseAttributeSolver):
    operator = Operators.EXISTS  # noqa: CCE003  # a static attribute

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:  # type:ignore[override]
        return vertex.get(attribute) is not None  # type:ignore[arg-type]  # due to attribute can be None



from __future__ import annotations

import logging
from abc import abstractmethod
from collections.abc import Iterable
from typing import List, Dict, Any, Callable, Optional

from checkov.common.typing import _SkippedCheck, _CheckResult
from checkov.common.util.type_forcers import force_list
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.multi_signature import MultiSignatureMeta, multi_signature


class BaseCheck(metaclass=MultiSignatureMeta):
    def __init__(
        self,
        name: str,
        id: str,
        categories: Iterable[CheckCategories],
        supported_entities: Iterable[str],
        block_type: str,
        bc_id: Optional[str] = None,
        guideline: Optional[str] = None,
    ) -> None:
        self.name = name
        self.id = id
        self.bc_id = bc_id
        self.categories = categories
        self.block_type = block_type
        self.path: str | None = None
        self.supported_entities = supported_entities
        self.logger = logging.getLogger("{}".format(self.__module__))
        self.evaluated_keys: List[str] = []
        self.entity_path = ""
        self.entity_type = ""
        self.guideline = guideline
        self.benchmarks: dict[str, list[str]] = {}
        self.severity = None
        self.bc_category = None
        if self.guideline:
            logging.debug(f'Found custom guideline for check {id}')
        self.details: List[str] = []

    def run(
        self,
        scanned_file: str,
        entity_configuration: Dict[str, List[Any]],
        entity_name: str,
        entity_type: str,
        skip_info: _SkippedCheck,
    ) -> _CheckResult:
        check_result: _CheckResult = {}
        if skip_info:
            check_result["result"] = CheckResult.SKIPPED
            check_result["suppress_comment"] = skip_info["suppress_comment"]
            message = 'File {}, {} "{}.{}" check "{}" Result: {}, Suppression comment: {} '.format(
                scanned_file,
                self.block_type,
                entity_type,
                entity_name,
                self.name,
                check_result,
                check_result["suppress_comment"],
            )
            self.logger.debug(message)
        else:
            try:
                self.evaluated_keys = []
                self.entity_path = f"{scanned_file}:{entity_type}:{entity_name}"
                check_result["result"] = self.scan_entity_conf(entity_configuration, entity_type)
                check_result["evaluated_keys"] = self.get_evaluated_keys()
                message = 'File {}, {}  "{}.{}" check "{}" Result: {} '.format(
                    scanned_file, self.block_type, entity_type, entity_name, self.name, check_result
                )
                self.logger.debug(message)

            except Exception:
                self.logger.error(
                    f"Failed to run check: {self.name} for configuration: {entity_configuration} at file: {scanned_file}"
                )
                raise
        return check_result

    @multi_signature()
    @abstractmethod
    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> CheckResult | tuple[CheckResult, dict[str, Any]]:
        raise NotImplementedError()

    @classmethod
    @scan_entity_conf.add_signature(args=["self", "conf"])
    def _scan_entity_conf_self_conf(
        cls, wrapped: Callable[..., CheckResult | tuple[CheckResult, dict[str, Any]]]
    ) -> Callable[..., CheckResult | tuple[CheckResult, dict[str, Any]]]:
        def wrapper(
            self: "BaseCheck", conf: Dict[str, Any], entity_type: Optional[str] = None
        ) -> CheckResult | tuple[CheckResult, dict[str, Any]]:
            # keep default argument for entity_type so old code, that doesn't set it, will work.
            return wrapped(self, conf)

        return wrapper

    def get_evaluated_keys(self) -> List[str]:
        """
        Retrieves the evaluated keys for the run's report. Child classes override the function and return the `expected_keys` instead.
        :return: List of the evaluated keys, as JSONPath syntax paths of the checked attributes
        """
        return force_list(self.evaluated_keys)

    def get_output_id(self, use_bc_ids: bool) -> str:
        return self.bc_id if self.bc_id and use_bc_ids else self.id



from typing import Optional, Any, Dict
from collections.abc import Sized
from checkov.common.checks_infra.solvers.attribute_solvers.base_attribute_solver import BaseAttributeSolver
from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.util.type_forcers import force_int


class LengthGreaterThanAttributeSolver(BaseAttributeSolver):
    operator = Operators.LENGTH_GREATER_THAN  # noqa: CCE003  # a static attribute

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:  # type:ignore[override]
        attr = vertex.get(attribute)  # type:ignore[arg-type]  # due to attribute can be None
        if attr is None:
            return False
        
        # if this value contains an underendered variable, then we cannot evaluate the check,
        # so return True (since we cannot return UNKNOWN)
        if self._is_variable_dependant(attr, vertex['source_']):
            return True

        value_int = force_int(self.value)

        if value_int is None:
            return False
        if isinstance(attr, Sized):
            return len(attr) > value_int

        return False



from typing import Union, List

from checkov.common.graph.graph_builder.variable_rendering.vertex_reference import VertexReference
from checkov.cloudformation.graph_builder.graph_components.block_types import BlockType


class CloudformationVertexReference(VertexReference[BlockType]):
    def __init__(self, block_type: Union[str, BlockType], sub_parts: List[str], origin_value: str) -> None:
        super().__init__(block_type, sub_parts, origin_value)

    @staticmethod
    def block_type_str_to_enum(block_type_str: str) -> BlockType:
        return BlockType().get(block_type_str)



import logging
import os
import re
from typing import Any, Union, Optional, List, Dict, Callable, TypeVar

# condition ? true_val : false_val -> (condition, true_val, false_val)
from checkov.common.util.type_forcers import force_int
from checkov.common.util.parser_utils import find_var_blocks
from checkov.terraform.graph_builder.variable_rendering.safe_eval_functions import evaluate

T = TypeVar("T", str, int, bool)

CONDITIONAL_EXPR = re.compile(r"([^\?]+)\?([^:]+)\:([^:]+)")

# %{ some_text }
DIRECTIVE_EXPR = re.compile(r"\%\{([^\}]*)\}")

COMPARE_REGEX = re.compile(r"^(?P<a>.+?)\s*(?P<operator>==|!=|>=|>|<=|<|&&|\|\|)\s*(?P<b>.+)$")
CHECKOV_RENDER_MAX_LEN = force_int(os.getenv("CHECKOV_RENDER_MAX_LEN", "10000"))


def evaluate_terraform(input_str: Any, keep_interpolations: bool = True) -> Any:
    if isinstance(input_str, str) and CHECKOV_RENDER_MAX_LEN and 0 < CHECKOV_RENDER_MAX_LEN < len(input_str):
        logging.info(f'Rendering was skipped for a {len(input_str)}-character-long string. If you wish to have it '
                     f'evaluated, please set the environment variable CHECKOV_RENDER_MAX_LEN '
                     f'to {str(len(input_str) + 1)} or to 0 to allow rendering of any length')
        return input_str
    evaluated_value = _try_evaluate(input_str)
    if type(evaluated_value) is not str:
        return input_str if callable(evaluated_value) else evaluated_value
    evaluated_value = evaluated_value.replace("\n", "")
    evaluated_value = evaluated_value.replace(",,", ",")

    # if we try to strip interpolations but that does not help evaluation, then we should add them back in the case that
    # the interpolated string is part of a substring, so it can be identified by the "is_variable_dependent" method.
    # For example, the value "abc-${var.x}-xyz" will not be identified as a variable if we remove the interpolation
    # However, if the full value is just an interpolated variable, like ${var.xyz}, then we can leave them off, because
    # it won't affect that method and breaks certain policies and other logic that was written in a specific way
    value_before_removing_interpolations = evaluated_value
    if not keep_interpolations:
        evaluated_value = remove_interpolation(evaluated_value)
    if '${' + evaluated_value + '}' == value_before_removing_interpolations:
        value_before_removing_interpolations = evaluated_value
    value_after_removing_interpolations = evaluated_value

    evaluated_value = evaluate_map(evaluated_value)
    evaluated_value = evaluate_list_access(evaluated_value)
    evaluated_value = strip_double_quotes(evaluated_value)
    evaluated_value = evaluate_directives(evaluated_value)
    evaluated_value = evaluate_conditional_expression(evaluated_value)
    evaluated_value = evaluate_compare(evaluated_value)
    evaluated_value = evaluate_json_types(evaluated_value)
    second_evaluated_value = _try_evaluate(evaluated_value)

    if callable(second_evaluated_value):
        return evaluated_value
    elif not keep_interpolations and second_evaluated_value == value_after_removing_interpolations:
        return value_before_removing_interpolations
    else:
        return second_evaluated_value


def _try_evaluate(input_str: Union[str, bool]) -> Any:
    try:
        return evaluate(input_str)
    except Exception:
        try:
            return evaluate(f'"{input_str}"')
        except Exception:
            return input_str


def replace_string_value(original_str: Any, str_to_replace: str, replaced_value: str, keep_origin: bool = True) -> Any:
    if original_str is None or type(original_str) not in (str, list):
        return original_str

    if type(original_str) is list:
        for i, item in enumerate(original_str):
            original_str[i] = replace_string_value(item, str_to_replace, replaced_value, keep_origin)
            if type(replaced_value) in [int, float, bool]:
                original_str[i] = evaluate_terraform(original_str[i])
            return original_str

    if str_to_replace not in original_str:
        return original_str if keep_origin else str_to_replace

    string_without_interpolation = remove_interpolation(original_str, str_to_replace, escape_unrendered=False)
    return string_without_interpolation.replace(str_to_replace, str(replaced_value))


def remove_interpolation(original_str: str, var_to_clean: Optional[str] = None, escape_unrendered=True) -> str:
    # get all variable references in string
    # remove from the string all ${} or '${}' occurrences
    var_blocks = find_var_blocks(original_str)
    var_blocks.reverse()
    for block in var_blocks:
        if (
            block.full_str.startswith("${")
            and block.full_str.endswith("}")
            and (not var_to_clean or block.var_only == var_to_clean)
        ):
            full_str_start = original_str.find(block.full_str)
            full_str_end = full_str_start + len(block.full_str)
            if (
                full_str_start > 0
                and full_str_end <= len(original_str) - 2
                and original_str[full_str_start - 1] == "'"
                and original_str[full_str_start - 1] == original_str[full_str_end]
                and "." in block.full_str
            ):
                # checking if ${} is wrapped with '' like : '${}'
                original_str = original_str[:full_str_start - 1] + block.full_str + original_str[full_str_end + 1:]
                if escape_unrendered:
                    block.var_only = f"'{block.var_only}'"
            original_str = original_str.replace(block.full_str, block.var_only)
    return original_str


def strip_double_quotes(input_str: str) -> str:
    if input_str.startswith('"') and input_str.endswith('"'):
        input_str = input_str[1:-1]
    return input_str


def evaluate_conditional_expression(input_str: str) -> str:
    variable_ref = re.match(re.compile(r"^\${(.*)}$"), input_str)
    if variable_ref:
        input_str = variable_ref.groups()[0]

    condition = re.match(CONDITIONAL_EXPR, input_str)
    while condition:
        groups = condition.groups()
        if len(groups) != 3:
            return input_str
        evaluated_condition = evaluate_terraform(groups[0])
        condition_substr = input_str[condition.start() : condition.end()]
        if convert_to_bool(evaluated_condition):
            true_val = str(evaluate_terraform(groups[1])).strip()
            input_str = input_str.replace(condition_substr, true_val)
        else:
            false_val = str(evaluate_terraform(groups[2])).strip()
            input_str = input_str.replace(condition_substr, false_val)
        condition = re.match(CONDITIONAL_EXPR, input_str)

    return input_str


def evaluate_compare(input_str: str) -> Union[str, bool]:
    """
    :param input_str: string like "a && b" (supported operators: ==, != , <, <=, >, >=, && , ||)
    :return: evaluation of the expression
    """
    if isinstance(input_str, str) and "for" not in input_str:
        match = re.search(COMPARE_REGEX, input_str)
        if match:
            compare_parts = match.groupdict()
            a = compare_parts.get("a")
            b = compare_parts.get("b")
            op = compare_parts.get("operator")
            if a and b and op:
                try:
                    return apply_binary_op(evaluate_terraform(a), evaluate_terraform(b), op)
                except TypeError or SyntaxError:
                    return input_str

    return input_str


def evaluate_json_types(input_str: Any) -> Any:
    # https://www.terraform.io/docs/language/functions/jsonencode.html
    if isinstance(input_str, str) and input_str.startswith("jsonencode("):
        return input_str.replace("true", "True").replace("false", "False").replace("null", "None")

    return input_str


def apply_binary_op(a: Optional[Union[str, int, bool]], b: Optional[Union[str, int, bool]], operator: str) -> bool:
    # apply the operator after verifying that a and b have the same type.
    operators: Dict[str, Callable[[T, T], bool]] = {
        "==": lambda a, b: a == b,
        "!=": lambda a, b: a != b,
        ">": lambda a, b: a > b,
        ">=": lambda a, b: a >= b,
        "<": lambda a, b: a < b,
        "<=": lambda a, b: a <= b,
        "&&": lambda a, b: a and b,
        "||": lambda a, b: a or b,
    }
    type_a = type(a)
    type_b = type(b)

    if type_a != type_b:
        try:
            temp_b = type_a(b)
            if isinstance(type_a, bool):
                temp_b = bool(convert_to_bool(b))
            return operators[operator](a, temp_b)
        except Exception:
            temp_a = type_b(a)
            if isinstance(type_b, bool):
                temp_a = bool(convert_to_bool(a))
            return operators[operator](temp_a, b)
    else:
        return operators[operator](a, b)


def evaluate_directives(input_str: str) -> str:
    if re.search(DIRECTIVE_EXPR, input_str) is None:
        return input_str

    # replace `%{if <BOOL>}%{true_val}%{else}%{false_val}%{endif}` pattern with `<BOOL> ? true_val : false_val`
    matching_directives = re.findall(DIRECTIVE_EXPR, input_str)
    if len(matching_directives) == 3:
        if (
            re.search(r"\bif\b", matching_directives[0])
            and re.search(r"\belse\b", matching_directives[1])
            and re.search(r"\bendif\b", matching_directives[2])
        ):
            split_by_directives = re.split(DIRECTIVE_EXPR, input_str)
            edited_str = ""
            for part in split_by_directives:
                if re.search(r"\bif\b", part):
                    part = part.replace("if", "%{") + " ? "
                    part = re.sub(r"\s", "", part)
                if re.search(r"\belse\b", part):
                    part = part.replace("else", ":")
                    part = re.sub(r"\s", "", part)
                if re.search(r"\bendif\b", part):
                    part = part.replace("endif", "}")
                    part = re.sub(r"\s", "", part)
                edited_str += part
            input_str = edited_str

    matching_directives = re.split(DIRECTIVE_EXPR, input_str)
    evaluated_string_parts = []
    for str_part in matching_directives:
        evaluated_string_parts.append(evaluate_terraform(str_part))

    # Handle evaluation results which are integer / boolean
    evaluated_string_parts = [v if isinstance(v, str) else str(v) for v in evaluated_string_parts]
    return "".join(evaluated_string_parts)


def evaluate_map(input_str: str) -> str:
    # first replace maps ":" with "="
    all_curly_brackets = find_brackets_pairs(input_str, "{", "}")
    if "=" in input_str:
        for curly_match in all_curly_brackets:
            curly_start = curly_match["start"]
            curly_end = curly_match["end"]
            replaced_matching_map = input_str[curly_start : curly_end + 1].replace("=", ":")
            input_str = input_str.replace(input_str[curly_start : curly_end + 1], replaced_matching_map)

    # find map access like {a: b}[a] and extract the right value - b
    all_square_brackets = find_brackets_pairs(input_str, "[", "]")

    curr_square_match = 0
    for curly_match in all_curly_brackets:
        curly_start = curly_match["start"]
        curly_end = curly_match["end"]
        for i in range(curr_square_match, len(all_square_brackets)):
            curr_square_match = i
            square_match = all_square_brackets[i]
            square_start = square_match["start"]
            square_end = square_match["end"]
            if square_start > curly_end and (
                square_start == curly_end + 1 or all(c == " " for c in input_str[curly_end + 1 : square_start])
            ):
                origin_match_str = input_str[curly_start : square_end + 1]
                map_access = input_str[square_start + 1 : square_end]
                if not map_access.startswith('"') and not map_access.endswith('"'):
                    origin_match_str = origin_match_str.replace(f"[{map_access}]", f'["{map_access}"]')
                evaluated = _try_evaluate(origin_match_str)
                if evaluated:
                    input_str = input_str.replace(input_str[curly_start : square_end + 1], str(evaluated))
                    break

    return input_str


def convert_to_bool(bool_str: Union[str, int]) -> Union[str, int, bool]:
    if bool_str in ["true", '"true"', "True", '"True"', 1, "1"]:
        return True
    elif bool_str in ["false", '"false"', "False", '"False"', 0, "0"]:
        return False
    else:
        return bool_str


def evaluate_list_access(input_str: str) -> str:
    # find list access like [a, b, c][0] and extract the right value - a

    all_square_brackets = find_brackets_pairs(input_str, "[", "]")
    prev_start = -1
    prev_end = -1
    for match in all_square_brackets:
        if (
            match["start"] == prev_end + 1 or all(c == " " for c in input_str[prev_end + 1 : match["start"]])
        ) and prev_start != -1:
            curr_str = input_str[match["start"] + 1 : match["end"]]
            if curr_str.isnumeric():
                evaluated = _try_evaluate(input_str[prev_start : match["end"] + 1])
                if evaluated:
                    input_str = input_str.replace(input_str[prev_start : match["end"] + 1], str(evaluated))
        prev_start = match["start"]
        prev_end = match["end"]

    return input_str


def find_brackets_pairs(input_str: str, starting: str, closing: str) -> List[Dict[str, int]]:
    brackets_pairs = [-1] * len(input_str)
    unmatched_open = []

    for i, c in enumerate(input_str):
        if c == starting:
            unmatched_open.append(i)
        elif c == closing and len(unmatched_open) > 0:
            brackets_pairs[unmatched_open[-1]] = i
            unmatched_open = unmatched_open[:-1]

    all_brackets = []
    for start, end in enumerate(brackets_pairs):
        if end != -1 and end - start > 1:
            all_brackets.append({"start": start, "end": end})
    return all_brackets



from checkov.terraform.checks.resource.alicloud.AbsRDSParameter import AbsRDSParameter


class RDSInstanceLogDisconnections(AbsRDSParameter):
    def __init__(self):
        super().__init__(check_id="CKV_ALI_36", parameter="log_disconnections")


check = RDSInstanceLogDisconnections()



from dataclasses import dataclass


@dataclass
class A:

    class_attribute: str = "skip and pass"

    @classmethod
    def _get_favicon_path(cls, object_name: str):
        pass

    def get_tabs_info(self):
        pass



import os

from checkov.common.bridgecrew.run_metadata.abstract_run_metadata_extractor import RunMetaDataExtractor


class BitbucketRunMetadataExtractor(RunMetaDataExtractor):
    def is_current_ci(self) -> bool:
        if os.getenv("BITBUCKET_BUILD_NUMBER", "") and os.getenv("CI", ""):
            return True
        return False

    def __init__(self) -> None:
        from_branch = os.getenv('BITBUCKET_BRANCH', "master")
        to_branch = os.getenv('BITBUCKET_PR_DESTINATION_BRANCH', "")
        pr_id = os.getenv("BITBUCKET_PR_ID", "")
        pr_url = ""
        commit_hash = os.getenv("BITBUCKET_COMMIT", "")
        repository_url = os.getenv("BITBUCKET_GIT_HTTP_ORIGIN","")
        commit_url = ""
        author_name = ""
        author_url = ""
        run_id = os.getenv("BITBUCKET_BUILD_NUMBER", "")
        run_url = ""

        super().__init__(from_branch=from_branch,
                         to_branch=to_branch,
                         pr_id=pr_id,
                         pr_url=pr_url,
                         commit_hash=commit_hash,
                         commit_url=commit_url,
                         author_name=author_name,
                         author_url=author_url,
                         run_id=run_id,
                         run_url=run_url,
                         repository_url=repository_url)


extractor = BitbucketRunMetadataExtractor()



#
# Example check
# This is checking yaml that looks like this example where
# Our resource is "jobs"
# It is of type ARRAY because there can be more than one that
# needs to be checked
#
# jobs:
#   unsecure-job:
#     name: job2
#     runs-on: ubuntu-latest
#     env:
#       ACTIONS_ALLOW_UNSECURE_COMMANDS: true
#     steps:
#       - name: unsecure-step2
#         run: |
#           echo "goo"
#   secure-job:
#     name: job3
#     runs-on: ubuntu-latest
#     env:
#       ACTIONS_ALLOW_UNSECURE_COMMANDS: false
#     run: |
#       echo "ok"
# 
from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult
# Import your base check
from checkov.example_runner.checks.base_example_runner_job_check import BaseExampleRunnerJobCheck

from checkov.yaml_doc.enums import BlockType


class ExampleCheckTrueFalse(BaseExampleRunnerJobCheck):
    def __init__(self) -> None:
        # Describe the check for the user
        name = "Ensure ACTIONS_ALLOW_UNSECURE_COMMANDS isn't true on environment variables on a job"
        # Give the check a unique id eg. CKV_TLA_24 where 
        #  CKV is standard for python checks
        #  TLA = Three letter acronym for your runner check type: GHA is GitHub Actions
        #  24 is the number in sequence of checks.  Must be unique!
        id = "CKV_GHA_1"
        super().__init__(
            name=name,
            id=id,
            # the block type tells the parse whether you'd expect one resource or many
            # options are ARRAY or OBJECT
            block_type=BlockType.ARRAY,
        )

    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> tuple[CheckResult, dict[str, Any]]:  # type:ignore[override]  # return type is different than the base class
        # The block type is passed as a data structure.  
        # Add logic to parse the structure for the misconfig
        # Remember to always return a PASSED or FAILED. 
        # It is easy to miss a result in complex logic
        if "env" not in conf:
            return CheckResult.PASSED, conf
        env_variables = conf.get("env", {})
        if env_variables.get("MY_ENV_IS_PASSED", False):
            return CheckResult.FAILED, conf
        return CheckResult.PASSED, conf

# Set this to your check name
check = ExampleCheckTrueFalse()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.type_forcers import force_int
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class RDSPostgreSQLLogFDWExtension(BaseResourceCheck):
    def __init__(self) -> None:
        # https://aws.amazon.com/security/security-bulletins/AWS-2022-004/
        name = "Ensure that RDS PostgreSQL instances use a non vulnerable version with the log_fdw extension (https://aws.amazon.com/security/security-bulletins/AWS-2022-004/)"
        id = "CKV_AWS_250"
        supported_resources = ("aws_rds_cluster", "aws_db_instance")
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        self.evaluated_keys = ["engine"]

        if conf.get("engine") == ["postgres"]:
            self.evaluated_keys.append("engine_version")

            engine_version = conf.get("engine_version")
            if engine_version and isinstance(engine_version, list) and isinstance(engine_version[0], str):
                version_parts = engine_version[0].split(".")
                if 1 < len(version_parts) <= 3:
                    major_version = force_int(version_parts[0])
                    minor_version = force_int(version_parts[1])

                    if major_version is None or minor_version is None:
                        return CheckResult.UNKNOWN

                    if major_version >= 14:
                        return CheckResult.PASSED
                    elif major_version == 13 and minor_version > 2:
                        return CheckResult.PASSED
                    elif major_version == 12 and minor_version > 6:
                        return CheckResult.PASSED
                    elif major_version == 11 and minor_version > 11:
                        return CheckResult.PASSED
                    elif major_version == 10 and minor_version > 16:
                        return CheckResult.PASSED
                    elif major_version == 9 and minor_version == 6:
                        # PostgreSQL pre 10 used following versioning major.major.minor
                        if len(version_parts) < 3:
                            return CheckResult.UNKNOWN

                        bugfix_version = force_int(version_parts[2])

                        if bugfix_version is None:
                            return CheckResult.UNKNOWN

                        if bugfix_version > 21:
                            return CheckResult.PASSED

                    # everything older is not recommended to use anyway
                    return CheckResult.FAILED
        elif conf.get("engine") == ["aurora-postgresql"]:
            self.evaluated_keys.append("engine_version")

            engine_version = conf.get("engine_version")
            if engine_version and isinstance(engine_version, list) and isinstance(engine_version[0], str):
                version_parts = engine_version[0].split(".")
                if len(version_parts) == 2:
                    major_version = force_int(version_parts[0])
                    minor_version = force_int(version_parts[1])

                    if major_version is None or minor_version is None:
                        return CheckResult.UNKNOWN

                    if major_version >= 12:
                        return CheckResult.PASSED
                    elif major_version == 11 and minor_version > 8:
                        return CheckResult.PASSED
                    elif major_version == 10 and minor_version > 13:
                        return CheckResult.PASSED

                    # older versions are not available for Aurora cluster
                    return CheckResult.FAILED

        # probably a non PostgreSQL instance or we couldn't render it correctly
        return CheckResult.UNKNOWN


check = RDSPostgreSQLLogFDWExtension()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck


class StorageAccountDefaultNetworkAccessDeny(BaseResourceCheck):
    def __init__(self):
        name = "Ensure default network access rule for Storage Accounts is set to deny"
        id = "CKV_AZURE_35"
        supported_resources = ['azurerm_storage_account', 'azurerm_storage_account_network_rules']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        '''
        Ensures a storage account is not widely accessible by default, using the default action in network rules configuration.
        Network Rules can be defined either directly on the azurerm_storage_account resource,
        or using the azurerm_storage_account_network_rules resource. If the latter is used, the check should be skipped for
        the azurerm_storage_account resource (which would return as failed).

        :param default_action:
        :return: Check Result
        '''
        network_conf = [conf]
        evaluated_key_prefix = ''
        if 'network_rules' in conf:
            network_conf = conf['network_rules']
            self.evaluated_keys = ['network_rules']
            evaluated_key_prefix = 'network_rules/[0]/'
        if 'default_action' in network_conf[0]:
            self.evaluated_keys = [f'{evaluated_key_prefix}default_action']
            if network_conf[0]['default_action'][0] == 'Deny':
                return CheckResult.PASSED
        return CheckResult.FAILED


check = StorageAccountDefaultNetworkAccessDeny()



from __future__ import annotations

from typing import Any

from checkov.common.util.type_forcers import force_int
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck


class BranchProtectionReviewNumTwo(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure at least two approving reviews for PRs"
        id = "CKV_GIT_5"
        supported_resources = ["github_branch_protection_v3", "github_branch_protection"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        if conf.get("required_pull_request_reviews") and isinstance(conf.get("required_pull_request_reviews"), list):
            review = conf.get("required_pull_request_reviews")[0]
            if review.get("required_approving_review_count") and isinstance(review.get("required_approving_review_count"),list):
                count = force_int(review.get("required_approving_review_count")[0])
                if count and count >= 2:
                    return CheckResult.PASSED
        self.evaluated_keys = ["required_pull_request_reviews/[0]/required_approving_review_count"]
        return CheckResult.FAILED


check = BranchProtectionReviewNumTwo()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class ImagePullPolicyAlways(BaseResourceCheck):

    def __init__(self):
        """
        Image pull policy should be set to always to ensure you get the correct image and imagePullSecrets are correct
        Default is 'IfNotPresent' unless image tag is omitted or :latest
        https://kubernetes.io/docs/concepts/configuration/overview/#container-images

        An admission controller could be used to enforce imagePullPolicy
        """
        name = "Image Pull Policy should be Always"
        id = "CKV_K8S_15"
        supported_resources = ["kubernetes_pod"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        spec = conf.get('spec', [None])[0]
        if isinstance(spec, dict) and spec:
            containers = spec.get("container")
            for idx, container in enumerate(containers):
                if not isinstance(container, dict):
                    return CheckResult.UNKNOWN

                if container.get("image_pull_policy"):
                    if container.get("image_pull_policy")[0] == "Always":
                        break
                else:
                    if container.get("image"):
                        name = container.get("image")[0]
                        if "latest" in name:
                            break
                self.evaluated_keys = [f'spec/[0]/container/[{idx}]']
                return CheckResult.FAILED
            return CheckResult.PASSED
        return CheckResult.FAILED


check = ImagePullPolicyAlways()



from typing import Any, Dict, Tuple

from checkov.common.models.enums import CheckResult
from checkov.github_actions.checks.base_github_action_check import BaseGithubActionsCheck
from checkov.yaml_doc.enums import BlockType
import re


class ReverseShellNetcat(BaseGithubActionsCheck):
    def __init__(self):
        name = "Suspicious use of netcat with IP address"
        id = "CKV_GHA_4"
        super().__init__(
            name=name,
            id=id,
            block_type=BlockType.ARRAY,
            supported_entities=['jobs','jobs.*.steps[]']
        )

    def scan_entity_conf(self, conf: Dict[str, Any]) -> Tuple[CheckResult, Dict[str, Any]]:
        if not isinstance(conf, dict):
            return CheckResult.UNKNOWN, conf
        run = conf.get("run", "")
        if re.search(r'(nc|netcat) (\d{1,3}).(\d{1,3}).(\d{1,3}).(\d{1,3})', run):
            return CheckResult.FAILED, conf
        return CheckResult.PASSED, conf


check = ReverseShellNetcat()



from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories

class IAMPassportAccountUsage(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure passport account is not used for assignment. Use service accounts and federated accounts where possible."
        id = "CKV_YC_24"
        categories = [CheckCategories.IAM]
        supported_resources = [
            "yandex_resourcemanager_folder_iam_binding",
            "yandex_resourcemanager_folder_iam_member",
            "yandex_resourcemanager_cloud_iam_binding",
            "yandex_resourcemanager_cloud_iam_member",
            "yandex_organizationmanager_organization_iam_binding",
            "yandex_organizationmanager_organization_iam_member"
        ]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def scan_resource_conf(self, conf):
        if self.entity_type == "yandex_resourcemanager_folder_iam_binding":
            for member in conf['members'][0]:
                if member.startswith("userAccount"):
                    return CheckResult.FAILED
            return CheckResult.PASSED
        if self.entity_type == "yandex_resourcemanager_folder_iam_member":
            if conf['member'][0].startswith("userAccount"):
                return CheckResult.FAILED
        if self.entity_type == "yandex_resourcemanager_cloud_iam_binding":
            for member in conf['members'][0]:
                if member.startswith("userAccount"):
                    return CheckResult.FAILED
            return CheckResult.PASSED
        if self.entity_type == "yandex_resourcemanager_cloud_iam_member":
            if conf['member'][0].startswith("userAccount"):
                return CheckResult.FAILED
        if self.entity_type == "yandex_organizationmanager_organization_iam_binding":
            for member in conf['members'][0]:
                if member.startswith("userAccount"):
                    return CheckResult.FAILED
            return CheckResult.PASSED
        if self.entity_type == "yandex_organizationmanager_organization_iam_member":
            if conf['member'][0].startswith("userAccount"):
                return CheckResult.FAILED
        return CheckResult.PASSED

scanner = IAMPassportAccountUsage()


from typing import Dict, List, Any

from checkov.common.util.type_forcers import force_list
from checkov.terraform.checks.data.base_check import BaseDataCheck
from checkov.common.models.enums import CheckResult, CheckCategories


class AdminPolicyDocument(BaseDataCheck):
    def __init__(self) -> None:
        name = 'Ensure IAM policies that allow full "*-*" administrative privileges are not created'
        id = "CKV_AWS_1"
        supported_data = ["aws_iam_policy_document"]
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_data=supported_data)

    def scan_data_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        """
            validates iam policy document
            https://learn.hashicorp.com/terraform/aws/iam-policy
        :param conf: aws_kms_key configuration
        :return: <CheckResult>
        """

        for statement in conf.get("statement", []):
            if not isinstance(statement, list):
                statement = [statement]
            for stmt in statement:
                if (
                    isinstance(stmt, dict)
                    and stmt.get("effect", ["Allow"]) == ["Allow"]
                    and stmt.get("actions")
                    and "*" in force_list(stmt["actions"][0])
                    and stmt.get("resources")
                    and "*" in force_list(stmt["resources"][0])
                ):
                    return CheckResult.FAILED

        return CheckResult.PASSED


check = AdminPolicyDocument()



from checkov.common.util.type_forcers import force_int

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class AbsGoogleComputeFirewallUnrestrictedIngress(BaseResourceCheck):
    def __init__(self, name, id, categories, supported_resources, port):
        super().__init__(name, id, categories, supported_resources)
        self.port = port

    def scan_resource_conf(self, conf):
        if 'allow' in conf:
            allow_blocks = conf['allow']
            self.evaluated_keys = ['allow']
            for block in allow_blocks:
                if isinstance(block, str):
                    self.evaluated_keys = [f'allow/[{allow_blocks.index(block)}]']
                    return CheckResult.UNKNOWN
                if 'ports' in block:
                    if self._is_port_in_range(block['ports']):
                        source_ranges_list = conf.get('source_ranges', [])
                        if source_ranges_list:
                            source_ranges = source_ranges_list[0]
                            if "0.0.0.0/0" in source_ranges:  # nosec
                                self.evaluated_keys = [f'allow/[{allow_blocks.index(block)}]/ports', 'source_ranges']
                                return CheckResult.FAILED
        return CheckResult.PASSED

    def _is_port_in_range(self, ports_list):
        if len(ports_list) == 0:
            return False
        if isinstance(ports_list[0], list):
            ports_list = ports_list[0]
        for port_range in ports_list:
            port = force_int(port_range)
            if port and self.port == port:
                return True
            if port is None and port_range and '-' in port_range:
                try:
                    [from_port, to_port] = port_range.split('-')
                    if int(from_port) <= self.port <= int(to_port):
                        return True
                except Exception:
                    return CheckResult.UNKNOWN
        return False



from __future__ import annotations

from collections.abc import Iterable
from typing import TYPE_CHECKING

from checkov.common.checks.base_check import BaseCheck
from checkov.dockerfile.registry import registry

if TYPE_CHECKING:
    from checkov.common.models.enums import CheckCategories


class BaseDockerfileCheck(BaseCheck):
    def __init__(
        self,
        name: str,
        id: str,
        categories: Iterable[CheckCategories],
        supported_instructions: Iterable[str],
        guideline: str | None = None,
    ) -> None:
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=supported_instructions,
            block_type="dockerfile",
            guideline=guideline,
        )
        self.supported_instructions = supported_instructions
        registry.register(self)



from __future__ import annotations

from dataclasses import dataclass, field


@dataclass
class ImageDetails:
    distro: str = ''
    distro_release: str = ''
    image_id: str = ''
    package_types: dict[str, str] = field(default_factory=dict)
    name: str | None = ''
    related_resource_id: str | None = ''



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class DefaultNamespace(BaseResourceCheck):
    def __init__(self):
        # CIS-1.5 5.7.4
        name = "The default namespace should not be used"
        id = "CKV_K8S_21"
        supported_resources = ["kubernetes_pod", "kubernetes_deployment", "kubernetes_daemonset",
                               "kubernetes_stateful_set", "kubernetes_replication_controller", "kubernetes_job",
                               "kubernetes_cron_job", "kubernetes_service", "kubernetes_secret",
                               "kubernetes_service_account", "kubernetes_role_binding", "kubernetes_config_map",
                               "kubernetes_ingress"]

        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf) -> CheckResult:
        if "metadata" not in conf:
            self.evaluated_keys = [""]
            return CheckResult.FAILED
        metadata = conf.get('metadata')[0]
        if metadata.get('namespace'):
            if metadata.get('namespace') == ["default"]:
                self.evaluated_keys = ['metadata/[0]/namespace']
                return CheckResult.FAILED
            return CheckResult.PASSED
        return CheckResult.FAILED


check = DefaultNamespace()



from checkov.common.bridgecrew.run_metadata.abstract_run_metadata_extractor import RunMetaDataExtractor


class DefaultRunMetadataExtractor(RunMetaDataExtractor):
    def is_current_ci(self) -> bool:
        return False

    def __init__(self) -> None:
        from_branch = ""
        to_branch = ""
        pr_id = ""
        pr_url = ""
        commit_hash = ""
        repository_url = ""
        commit_url = ""
        author_name = ""
        author_url = ""
        run_id = ""
        run_url = ""

        super().__init__(from_branch=from_branch,
                         to_branch=to_branch,
                         pr_id=pr_id,
                         pr_url=pr_url,
                         commit_hash=commit_hash,
                         commit_url=commit_url,
                         author_name=author_name,
                         author_url=author_url,
                         run_id=run_id,
                         run_url=run_url,
                         repository_url=repository_url)


extractor = DefaultRunMetadataExtractor()



from __future__ import annotations

from typing import Any

from checkov.argo_workflows.checks.base_argo_workflows_check import BaseArgoWorkflowsCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.yaml_doc.enums import BlockType


class DefaultServiceAccount(BaseArgoWorkflowsCheck):
    def __init__(self) -> None:
        name = "Ensure Workflow pods are not using the default ServiceAccount"
        id = "CKV_ARGO_1"
        super().__init__(
            name=name,
            id=id,
            categories=(CheckCategories.IAM,),
            supported_entities=("spec",),
            block_type=BlockType.OBJECT,
        )

    def scan_conf(self, conf: dict[str, Any]) -> tuple[CheckResult, dict[str, Any]]:
        if "serviceAccountName" in conf.keys() and conf["serviceAccountName"] != "default":
            return CheckResult.PASSED, conf

        return CheckResult.FAILED, conf


check = DefaultServiceAccount()



from typing import Union, List, Dict, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck
from checkov.common.util.type_forcers import force_list
import re

INTERNET_ADDRESSES = ("*", "0.0.0.0", "<nw>/0", "/0", "internet", "any")  # nosec
PORT_RANGE = re.compile(r"\d+-\d+")


class NSGRulePortAccessRestricted(BaseResourceCheck):
    def __init__(self, name: str, check_id: str, port: int) -> None:
        supported_resources = ("azurerm_network_security_rule", "azurerm_network_security_group")
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=check_id, categories=categories, supported_resources=supported_resources)
        self.port = port

    def is_port_in_range(self, ports: Union[int, str, List[Union[int, str]]]) -> bool:
        for range in force_list(ports):
            str_range = str(range)
            if re.match(PORT_RANGE, str_range):
                start, end = int(range.split("-")[0]), int(range.split("-")[1])
                if start <= self.port <= end:
                    return True
            if str_range in (str(self.port), "*"):
                return True
        return False

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        if "dynamic" in conf:
            self.evaluated_keys = ["dynamic"]
            return CheckResult.UNKNOWN

        rule_confs = [conf]
        evaluated_key_prefix = ""
        if "security_rule" in conf:
            rule_confs = conf["security_rule"]
            self.evaluated_keys = ["security_rule"]
            evaluated_key_prefix = "security_rule/"

        for rule_conf in rule_confs:
            if not isinstance(rule_conf, dict):
                return CheckResult.UNKNOWN

            access = rule_conf.get("access")
            direction = rule_conf.get("direction")
            protocol = rule_conf.get("protocol")
            destination_port_range = rule_conf.get("destination_port_range")
            destination_port_ranges = rule_conf.get("destination_port_ranges")
            source_address_prefix = rule_conf.get("source_address_prefix")
            source_address_prefixes = rule_conf.get("source_address_prefixes")

            if (
                access
                and access[0].lower() == "allow"
                and direction
                and direction[0].lower() == "inbound"
                and protocol
                and protocol[0].lower() in ("tcp", "*")
                and (
                    (
                        destination_port_range
                        and self.is_port_in_range(destination_port_range[0])  # fmt: skip
                    )
                    or (
                        destination_port_ranges
                        and destination_port_ranges[0]
                        and any(self.is_port_in_range(range) for range in destination_port_ranges[0])
                    )
                )
                and (
                    (
                        source_address_prefix
                        and isinstance(source_address_prefix[0], str)
                        and source_address_prefix[0].lower() in INTERNET_ADDRESSES  # fmt: skip
                    )
                    or (
                        source_address_prefixes
                        and source_address_prefixes[0]
                        and isinstance(source_address_prefixes[0], list)
                        and any((isinstance(prefix, str) and prefix.lower()) in INTERNET_ADDRESSES for prefix in
                                source_address_prefixes[0])
                    )
                )
            ):
                evaluated_key_prefix = (
                    f"{evaluated_key_prefix}[{rule_confs.index(rule_conf)}]/" if evaluated_key_prefix else ""
                )
                self.evaluated_keys = [
                    f"{evaluated_key_prefix}access",
                    f"{evaluated_key_prefix}direction",
                    f"{evaluated_key_prefix}protocol",
                    f"{evaluated_key_prefix}destination_port_range",
                    f"{evaluated_key_prefix}destination_port_ranges",
                    f"{evaluated_key_prefix}source_address_prefix",
                    f"{evaluated_key_prefix}source_address_prefixes",
                ]
                return CheckResult.FAILED

        return CheckResult.PASSED



from typing import Dict

from checkov.common.bridgecrew.severities import BcSeverities
from cyclonedx.model.vulnerability import VulnerabilitySeverity
from checkov.common.output.report import CheckType
from cyclonedx.output import SchemaVersion

SCA_CHECKTYPES = (CheckType.SCA_PACKAGE, CheckType.SCA_IMAGE)

PURL_TYPE_MAVEN = "maven"

DEFAULT_CYCLONE_SCHEMA_VERSION = SchemaVersion.V1_4

CYCLONE_SCHEMA_VERSION: Dict[str, str] = {
    "1.4": DEFAULT_CYCLONE_SCHEMA_VERSION,
    "1.3": SchemaVersion.V1_3,
    "1.2": SchemaVersion.V1_2,
    "1.1": SchemaVersion.V1_1,
    "1.0": SchemaVersion.V1_0
}

FILE_NAME_TO_PURL_TYPE = {
    "build.gradle": "maven",
    "build.gradle.kts": PURL_TYPE_MAVEN,
    "composer.json": "composer",
    "Gemfile": "gem",
    "go.mod": "golang",
    "go.sum": "golang",
    "package.json": "npm",
    "package-lock.json": "npm",
    "Pipfile": "pypi",
    "Pipfile.lock": "pypi",
    "pom.xml": "maven",
    "requirements.txt": "pypi",
    "yarn.lock": "npm"
}

IMAGE_DISTRO_TO_PURL_TYPE = {
    'Debian': 'deb',
    'Red': 'rpm',
    'CentOS': 'rpm',
    'Fedora': 'rpm',
    'openSUSE': 'rpm',
    'AlmaLinux': 'rpm',
    'Asianux': 'rpm',
    'ClearOS': 'rpm',
    'Fermi': 'rpm',
    'Miracle': 'rpm',
    'Oracle': 'rpm',
    'Rocky': 'rpm',
    'Scientific': 'rpm',
    'Amazon': 'rpm',
    'SUSE': 'rpm',
    'GeckoLinux': 'rpm',
    'Mandriva': 'rpm',
    'Mageia': 'rpm',
    'ROSA': 'rpm',
    'OpenMandriva': 'rpm',
    'Unity': 'rpm',
    'PCLinuxOS': 'rpm',
    'Vine': 'rpm',
    'ALT': 'rpm',
    'Caldera': 'rpm',
    'cAos': 'rpm',
    'Turbolinux': 'rpm',
    'Astra': 'deb',
    'Bharat': 'deb',
    'Canaima': 'deb',
    'Corel': 'deb',
    'CrunchBang': 'deb',
    'Deepin': 'deb',
    'Devuan': 'deb',
    'Dreamlinux': 'deb',
    'Emdebian': 'deb',
    'Finnix': 'deb',
    'gNewSense': 'deb',
    'Gnoppix': 'deb',
    'grml': 'deb',
    'HandyLinux': 'deb',
    'Kanotix': 'deb',
    'Knoppix': 'deb',
    'Kurumin': 'deb',
    'LEAF': 'deb',
    'LiMux': 'deb',
    'LMDE': 'deb',
    'Maemo': 'deb',
    'MEPIS': 'deb',
    'MintPPC': 'deb',
    'Musix': 'deb',
    'NepaLinux': 'deb',
    'OpenZaurus': 'deb',
    'Pardus': 'deb',
    'PelicanHPC': 'deb',
    'Q4OS': 'deb',
    'Raspberry': 'deb',
    'Sacix': 'deb',
    'Skolelinux': 'deb',
    'Slax (since': 'deb',
    'SolydXK': 'deb',
    'SparkyLinux': 'deb',
    'Sunwah': 'deb',
    'The': 'deb',
    'TurnKey': 'deb',
    'Twister': 'deb',
    'Ubuntu': 'deb',
    'Univention': 'deb',
    'Webconverger': 'deb',
    'Vyatta': 'deb',
    'VyOS': 'deb',
    'BackTrack': 'deb',
    'gLinux': 'deb',
    'Kali': 'deb',
    'Parsix': 'deb',
    'EOL[44]': 'deb',
    'PureOS': 'deb',
    'Parrot': 'deb',
    'antiX': 'deb',
    'MX': 'deb',
    'Damn': 'deb',
    'Feather': 'deb',
    'Hikarunix': 'deb',
    'Kubuntu': 'deb',
    'Lubuntu': 'deb',
    'Xubuntu': 'deb',
    'Edubuntu': 'deb',
    'Gobuntu': 'deb',
    'Mythbuntu': 'deb',
    'BackBox': 'deb',
    'BackSlash': 'deb',
    'Bodhi': 'deb',
    'Cub': 'deb',
    'DNALinux (based': 'deb',
    'dyne:bolic': 'deb',
    'EasyPeasy': 'deb',
    'Eeebuntu': 'deb',
    'Element': 'deb',
    'elementary': 'deb',
    'Emmabuntüs': 'deb',
    'GalliumOS': 'deb',
    'GendBuntu': 'deb',
    'Goobuntu': 'deb',
    'gOS': 'deb',
    'Joli': 'deb',
    'Karoshi': 'deb',
    'KDE': 'deb',
    'Linux': 'deb',
    'LinuxMCE': 'deb',
    'LinuxTLE': 'deb',
    'LliureX': 'deb',
    'LXLE': 'deb',
    'MAX': 'deb',
    'Molinux': 'deb',
    'Netrunner': 'deb',
    'Nova': 'deb',
    'OpenGEU': 'deb',
    'Peppermint': 'deb',
    'Pinguy': 'deb',
    'Pop!': 'deb',
    'Poseidon': 'deb',
    'Sabily': 'deb',
    'Trisquel': 'deb',
    'UberStudent': 'deb',
    'Ututo': 'deb',
    'Vinux': 'deb',
    'Zorin': 'deb'
}

TWISTCLI_PACKAGE_TYPE_TO_PURL_TYPE = {
    'python' : 'pypi',
    'nodejs' : 'npm',
    'jar' : 'maven',
    'rust': 'cargo'
}

BC_SEVERITY_TO_CYCLONEDX_LEVEL = {
    BcSeverities.CRITICAL: VulnerabilitySeverity.CRITICAL,
    BcSeverities.HIGH: VulnerabilitySeverity.HIGH,
    BcSeverities.MEDIUM: VulnerabilitySeverity.MEDIUM,
    BcSeverities.LOW: VulnerabilitySeverity.LOW,
    BcSeverities.NONE: VulnerabilitySeverity.NONE,
}



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class CosmosDBAccountsRestrictedAccess(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Cosmos DB accounts have restricted access"
        id = "CKV_AZURE_99"
        supported_resources = ['azurerm_cosmosdb_account']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'public_network_access_enabled' not in conf or conf['public_network_access_enabled'][0]:
            self.evaluated_keys = ['public_network_access_enabled']
            if 'is_virtual_network_filter_enabled' in conf and conf['is_virtual_network_filter_enabled'][0]:
                self.evaluated_keys.append('is_virtual_network_filter_enabled')
                if 'virtual_network_rule' in conf and conf['virtual_network_rule'][0]:
                    self.evaluated_keys.append('virtual_network_rule')
                    return CheckResult.PASSED
                elif 'ip_range_filter' in conf and conf['ip_range_filter'][0]:
                    self.evaluated_keys.append('ip_range_filter')
                    return CheckResult.PASSED
            return CheckResult.FAILED
        return CheckResult.PASSED


check = CosmosDBAccountsRestrictedAccess()



from typing import Optional, Any, Dict

from checkov.common.graph.checks_infra.enums import Operators
from .is_true_attribute_solver import IsTrueAttributeSolver


class IsFalseAttributeSolver(IsTrueAttributeSolver):
    operator = Operators.IS_FALSE  # noqa: CCE003  # a static attribute

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:  # type:ignore[override]
        return not super()._get_operation(vertex, attribute)



from __future__ import annotations

from typing import TYPE_CHECKING

from hcl2 import START_LINE, END_LINE

from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.images.image_referencer import Image
from checkov.common.util.data_structures_utils import find_in_dict
from checkov.common.util.type_forcers import force_list, extract_json

if TYPE_CHECKING:
    from networkx import DiGraph

SUPPORTED_AWS_IMAGE_RESOURCE_TYPES = {
    "aws_apprunner_service",
    "aws_batch_job_definition",
    "aws_codebuild_project",
    "aws_ecs_task_definition",
    "aws_lightsail_container_service_deployment_version",
}


def extract_images_from_aws_resources(graph_connector: DiGraph) -> list[Image]:
    images = []
    image_names = []

    resource_nodes = [
        node
        for node, resource_type in graph_connector.nodes(data=CustomAttributes.RESOURCE_TYPE)
        if resource_type and resource_type in SUPPORTED_AWS_IMAGE_RESOURCE_TYPES
    ]

    supported_resources_graph = graph_connector.subgraph(resource_nodes)

    for _, resource in supported_resources_graph.nodes(data=True):
        if resource[CustomAttributes.RESOURCE_TYPE] == "aws_apprunner_service":
            image_repo = find_in_dict(input_dict=resource, key_path="source_configuration/image_repository")
            if isinstance(image_repo, dict):
                repo_type = image_repo.get("image_repository_type")
                name = image_repo.get("image_identifier")
                if name and isinstance(name, str) and repo_type == "ECR_PUBLIC":
                    image_names.append(name)

        elif resource[CustomAttributes.RESOURCE_TYPE] == "aws_batch_job_definition":
            properties = extract_json(resource.get("container_properties"))
            if isinstance(properties, dict):
                name = properties.get("image")
                if name and isinstance(name, str):
                    image_names.append(name)
                node_range = find_in_dict(
                    input_dict=properties,
                    key_path="nodeProperties/nodeRangeProperties"
                )
                if isinstance(node_range, list):
                    for node in node_range:
                        name = find_in_dict(input_dict=node, key_path="container/image")
                        if name and isinstance(name, str):
                            image_names.append(name)

        elif resource[CustomAttributes.RESOURCE_TYPE] == "aws_codebuild_project":
            name = find_in_dict(input_dict=resource, key_path="environment/image")
            if name and isinstance(name, str) and not name.startswith("aws/codebuild/"):  # AWS provided images have an internal identifier
                image_names.append(name)

        elif resource[CustomAttributes.RESOURCE_TYPE] == "aws_ecs_task_definition":
            definitions = extract_json(resource.get("container_definitions"))
            if isinstance(definitions, list):
                for definition in definitions:
                    name = definition.get("image")
                    if name and isinstance(name, str):
                        image_names.append(name)

        elif resource[CustomAttributes.RESOURCE_TYPE] == "aws_lightsail_container_service_deployment_version":
            containers = resource.get("container")
            if containers:
                for container in force_list(containers):
                    name = container.get("image")
                    if name and isinstance(name, str):
                        image_names.append(name)

        for name in image_names:
            images.append(
                Image(
                    file_path=resource[CustomAttributes.FILE_PATH],
                    name=name,
                    start_line=resource[START_LINE],
                    end_line=resource[END_LINE],
                )
            )

    return images
